@@ -87,9 +87,13 @@ macro_rules! jni_map_error_with_env {
         match $result {
             Ok(result) => $crate::jni_bridge::datafusion::error::Result::Ok(result),
             Err($crate::jni_bridge::jni::errors::Error::JavaException) => {
-                let ex = $env.exception_occurred().unwrap();
-                $env.exception_describe().unwrap();
-                $env.exception_clear().unwrap();
+                let ex = $env
+                    .exception_occurred()
+                    .expect("failed to obtain pending Java execption object");
+                $env.exception_describe()
+                    .expect("failed to print Java exception to stderr");
+                $env.exception_clear()
+                    .expect("failed to clear pending Java exception");
                 let message_obj = $env
                     .call_method_unchecked(
                         ex,
@@ -102,13 +106,13 @@ macro_rules! jni_map_error_with_env {
                             .clone(),
                         &[],
                     )
-                    .unwrap()
+                    .expect("call Java Throwable.toString() failed")
                     .l()
-                    .unwrap();
+                    .expect("expected object return from Throwable.toString()");
                 let message = $env
                     .get_string(message_obj.into())
                     .map(|s| String::from(s))
-                    .unwrap();
+                    .expect("failed to read Throwable.toString() result as Java string");
 
                 Err(
                     $crate::jni_bridge::datafusion::error::DataFusionError::External(
@@ -439,11 +443,10 @@ pub struct JavaClasses<'a> {
     pub cSparkFileSegment: SparkFileSegment<'a>,
     pub cSparkSQLMetric: SparkSQLMetric<'a>,
     pub cSparkMetricNode: SparkMetricNode<'a>,
-    pub cSparkUDFWrapperContext: SparkUDFWrapperContext<'a>,
+    pub cSparkAuronUDFWrapperContext: SparkAuronUDFWrapperContext<'a>,
     pub cSparkUDAFWrapperContext: SparkUDAFWrapperContext<'a>,
     pub cSparkUDTFWrapperContext: SparkUDTFWrapperContext<'a>,
     pub cSparkUDAFMemTracker: SparkUDAFMemTracker<'a>,
-    pub cAuronConf: AuronConf<'a>,
     pub cAuronRssPartitionWriterBase: AuronRssPartitionWriterBase<'a>,
     pub cAuronCallNativeWrapper: AuronCallNativeWrapper<'a>,
     pub cAuronOnHeapSpillManager: AuronOnHeapSpillManager<'a>,
@@ -453,6 +456,8 @@ pub struct JavaClasses<'a> {
     pub cAuronFSDataInputWrapper: AuronFSDataInputWrapper<'a>,
     pub cAuronFSDataOutputWrapper: AuronFSDataOutputWrapper<'a>,
     pub cAuronJsonFallbackWrapper: AuronJsonFallbackWrapper<'a>,
+
+    pub cAuronUDFWrapperContext: AuronUDFWrapperContext<'a>,
 }
 
 #[allow(clippy::non_send_fields_in_send_ty)]
@@ -503,11 +508,10 @@ impl JavaClasses<'static> {
                 cSparkFileSegment: SparkFileSegment::new(env)?,
                 cSparkSQLMetric: SparkSQLMetric::new(env)?,
                 cSparkMetricNode: SparkMetricNode::new(env)?,
-                cSparkUDFWrapperContext: SparkUDFWrapperContext::new(env)?,
+                cSparkAuronUDFWrapperContext: SparkAuronUDFWrapperContext::new(env)?,
                 cSparkUDAFWrapperContext: SparkUDAFWrapperContext::new(env)?,
                 cSparkUDTFWrapperContext: SparkUDTFWrapperContext::new(env)?,
                 cSparkUDAFMemTracker: SparkUDAFMemTracker::new(env)?,
-                cAuronConf: AuronConf::new(env)?,
                 cAuronRssPartitionWriterBase: AuronRssPartitionWriterBase::new(env)?,
                 cAuronCallNativeWrapper: AuronCallNativeWrapper::new(env)?,
                 cAuronOnHeapSpillManager: AuronOnHeapSpillManager::new(env)?,
@@ -517,6 +521,8 @@ impl JavaClasses<'static> {
                 cAuronFSDataInputWrapper: AuronFSDataInputWrapper::new(env)?,
                 cAuronFSDataOutputWrapper: AuronFSDataOutputWrapper::new(env)?,
                 cAuronJsonFallbackWrapper: AuronJsonFallbackWrapper::new(env)?,
+
+                cAuronUDFWrapperContext: AuronUDFWrapperContext::new(env)?,
             };
             log::info!("Initializing JavaClasses finished");
             Ok(java_classes)
@@ -536,7 +542,7 @@ impl JavaClasses<'static> {
 
     pub fn get() -> &'static JavaClasses<'static> {
         unsafe {
-            // safety: JNI_JAVA_CLASSES must be initialized frist
+            // safety: JNI_JAVA_CLASSES must be initialized first
             JNI_JAVA_CLASSES.get_unchecked()
         }
     }
@@ -549,18 +555,16 @@ pub struct JniBridge<'a> {
     pub method_getContextClassLoader_ret: ReturnType,
     pub method_setContextClassLoader: JStaticMethodID,
     pub method_setContextClassLoader_ret: ReturnType,
-    pub method_getSparkEnvConfAsString: JStaticMethodID,
-    pub method_getSparkEnvConfAsString_ret: ReturnType,
     pub method_getResource: JStaticMethodID,
     pub method_getResource_ret: ReturnType,
-    pub method_getTaskContext: JStaticMethodID,
-    pub method_getTaskContext_ret: ReturnType,
+    pub method_getThreadContext: JStaticMethodID,
+    pub method_getThreadContext_ret: ReturnType,
+    pub method_setThreadContext: JStaticMethodID,
+    pub method_setThreadContext_ret: ReturnType,
     pub method_getTaskOnHeapSpillManager: JStaticMethodID,
     pub method_getTaskOnHeapSpillManager_ret: ReturnType,
     pub method_isTaskRunning: JStaticMethodID,
     pub method_isTaskRunning_ret: ReturnType,
-    pub method_isDriverSide: JStaticMethodID,
-    pub method_isDriverSide_ret: ReturnType,
     pub method_openFileAsDataInputWrapper: JStaticMethodID,
     pub method_openFileAsDataInputWrapper_ret: ReturnType,
     pub method_createFileAsDataOutputWrapper: JStaticMethodID,
@@ -571,11 +575,21 @@ pub struct JniBridge<'a> {
     pub method_getTotalMemoryLimited_ret: ReturnType,
     pub method_getDirectWriteSpillToDiskFile: JStaticMethodID,
     pub method_getDirectWriteSpillToDiskFile_ret: ReturnType,
-    pub method_initNativeThread: JStaticMethodID,
-    pub method_initNativeThread_ret: ReturnType,
+    pub method_getAuronUDFWrapperContext: JStaticMethodID,
+    pub method_getAuronUDFWrapperContext_ret: ReturnType,
+    pub method_intConf: JStaticMethodID,
+    pub method_intConf_ret: ReturnType,
+    pub method_longConf: JStaticMethodID,
+    pub method_longConf_ret: ReturnType,
+    pub method_doubleConf: JStaticMethodID,
+    pub method_doubleConf_ret: ReturnType,
+    pub method_booleanConf: JStaticMethodID,
+    pub method_booleanConf_ret: ReturnType,
+    pub method_stringConf: JStaticMethodID,
+    pub method_stringConf_ret: ReturnType,
 }
 impl<'a> JniBridge<'a> {
-    pub const SIG_TYPE: &'static str = "org/apache/spark/sql/auron/JniBridge";
+    pub const SIG_TYPE: &'static str = "org/apache/auron/jni/JniBridge";
 
     pub fn new(env: &JNIEnv<'a>) -> JniResult<JniBridge<'a>> {
         let class = get_global_jclass(env, Self::SIG_TYPE)?;
@@ -593,46 +607,44 @@ impl<'a> JniBridge<'a> {
                 "(Ljava/lang/ClassLoader;)V",
             )?,
             method_setContextClassLoader_ret: ReturnType::Primitive(Primitive::Void),
-            method_getSparkEnvConfAsString: env.get_static_method_id(
-                class,
-                "getSparkEnvConfAsString",
-                "(Ljava/lang/String;)Ljava/lang/String;",
-            )?,
-            method_getSparkEnvConfAsString_ret: ReturnType::Object,
             method_getResource: env.get_static_method_id(
                 class,
                 "getResource",
                 "(Ljava/lang/String;)Ljava/lang/Object;",
             )?,
             method_getResource_ret: ReturnType::Object,
-            method_getTaskContext: env.get_static_method_id(
+            method_getThreadContext: env.get_static_method_id(
                 class,
-                "getTaskContext",
-                "()Lorg/apache/spark/TaskContext;",
+                "getThreadContext",
+                "()Ljava/lang/Object;",
             )?,
-            method_getTaskContext_ret: ReturnType::Object,
+            method_getThreadContext_ret: ReturnType::Object,
+            method_setThreadContext: env.get_static_method_id(
+                class,
+                "setThreadContext",
+                "(Ljava/lang/Object;)V",
+            )?,
+            method_setThreadContext_ret: ReturnType::Primitive(Primitive::Void),
             method_getTaskOnHeapSpillManager: env.get_static_method_id(
                 class,
                 "getTaskOnHeapSpillManager",
-                "()Lorg/apache/spark/sql/auron/memory/OnHeapSpillManager;",
+                "()Lorg/apache/auron/memory/OnHeapSpillManager;",
             )?,
             method_getTaskOnHeapSpillManager_ret: ReturnType::Object,
             method_isTaskRunning: env.get_static_method_id(class, "isTaskRunning", "()Z")?,
             method_isTaskRunning_ret: ReturnType::Primitive(Primitive::Boolean),
-            method_isDriverSide: env.get_static_method_id(class, "isDriverSide", "()Z")?,
             method_openFileAsDataInputWrapper: env.get_static_method_id(
                 class,
                 "openFileAsDataInputWrapper",
-                "(Lorg/apache/hadoop/fs/FileSystem;Ljava/lang/String;)Lorg/apache/spark/auron/FSDataInputWrapper;",
+                "(Lorg/apache/hadoop/fs/FileSystem;Ljava/lang/String;)Lorg/apache/auron/hadoop/fs/FSDataInputWrapper;",
             )?,
             method_openFileAsDataInputWrapper_ret: ReturnType::Object,
             method_createFileAsDataOutputWrapper: env.get_static_method_id(
                 class,
                 "createFileAsDataOutputWrapper",
-                "(Lorg/apache/hadoop/fs/FileSystem;Ljava/lang/String;)Lorg/apache/spark/auron/FSDataOutputWrapper;",
+                "(Lorg/apache/hadoop/fs/FileSystem;Ljava/lang/String;)Lorg/apache/auron/hadoop/fs/FSDataOutputWrapper;",
             )?,
             method_createFileAsDataOutputWrapper_ret: ReturnType::Object,
-            method_isDriverSide_ret: ReturnType::Primitive(Primitive::Boolean),
             method_getDirectMemoryUsed: env.get_static_method_id(
                 class,
                 "getDirectMemoryUsed",
@@ -651,12 +663,43 @@ impl<'a> JniBridge<'a> {
                 "()Ljava/lang/String;",
             )?,
             method_getDirectWriteSpillToDiskFile_ret: ReturnType::Object,
-            method_initNativeThread: env.get_static_method_id(
+
+            method_getAuronUDFWrapperContext: env.get_static_method_id(
                 class,
-                "initNativeThread",
-                "(Ljava/lang/ClassLoader;Lorg/apache/spark/TaskContext;)V",
+                "getAuronUDFWrapperContext",
+                "(Ljava/nio/ByteBuffer;)Lorg/apache/auron/functions/AuronUDFWrapperContext;",
             )?,
-            method_initNativeThread_ret: ReturnType::Primitive(Primitive::Void),
+            method_getAuronUDFWrapperContext_ret: ReturnType::Object,
+            method_intConf: env.get_static_method_id(
+                class,
+                "intConf",
+                "(Ljava/lang/String;)I",
+            )?,
+            method_intConf_ret: ReturnType::Primitive(Primitive::Int),
+            method_longConf: env.get_static_method_id(
+                class,
+                "longConf",
+                "(Ljava/lang/String;)J",
+            )?,
+            method_longConf_ret: ReturnType::Primitive(Primitive::Long),
+            method_doubleConf: env.get_static_method_id(
+                class,
+                "doubleConf",
+                "(Ljava/lang/String;)D",
+            )?,
+            method_doubleConf_ret: ReturnType::Primitive(Primitive::Double),
+            method_booleanConf: env.get_static_method_id(
+                class,
+                "booleanConf",
+                "(Ljava/lang/String;)Z",
+            )?,
+            method_booleanConf_ret: ReturnType::Primitive(Primitive::Boolean),
+            method_stringConf: env.get_static_method_id(
+                class,
+                "stringConf",
+                "(Ljava/lang/String;)Ljava/lang/String;",
+            )?,
+            method_stringConf_ret: ReturnType::Object,
         })
     }
 }
@@ -1097,7 +1140,7 @@ pub struct SparkMetricNode<'a> {
     pub method_add_ret: ReturnType,
 }
 impl<'a> SparkMetricNode<'a> {
-    pub const SIG_TYPE: &'static str = "org/apache/spark/sql/auron/MetricNode";
+    pub const SIG_TYPE: &'static str = "org/apache/auron/metric/SparkMetricNode";
 
     pub fn new(env: &JNIEnv<'a>) -> JniResult<SparkMetricNode<'a>> {
         let class = get_global_jclass(env, Self::SIG_TYPE)?;
@@ -1106,7 +1149,7 @@ impl<'a> SparkMetricNode<'a> {
             method_getChild: env.get_method_id(
                 class,
                 "getChild",
-                "(I)Lorg/apache/spark/sql/auron/MetricNode;",
+                "(I)Lorg/apache/auron/metric/MetricNode;",
             )?,
             method_getChild_ret: ReturnType::Object,
             method_add: env.get_method_id(class, "add", "(Ljava/lang/String;J)V")?,
@@ -1115,58 +1158,6 @@ impl<'a> SparkMetricNode<'a> {
     }
 }
 
-#[allow(non_snake_case)]
-pub struct AuronConf<'a> {
-    pub class: JClass<'a>,
-    pub method_booleanConf: JStaticMethodID,
-    pub method_booleanConf_ret: ReturnType,
-    pub method_intConf: JStaticMethodID,
-    pub method_intConf_ret: ReturnType,
-    pub method_longConf: JStaticMethodID,
-    pub method_longConf_ret: ReturnType,
-    pub method_doubleConf: JStaticMethodID,
-    pub method_doubleConf_ret: ReturnType,
-    pub method_stringConf: JStaticMethodID,
-    pub method_stringConf_ret: ReturnType,
-}
-
-impl<'a> AuronConf<'_> {
-    pub const SIG_TYPE: &'static str = "org/apache/spark/sql/auron/AuronConf";
-
-    pub fn new(env: &JNIEnv<'a>) -> JniResult<AuronConf<'a>> {
-        let class = get_global_jclass(env, Self::SIG_TYPE)?;
-        Ok(AuronConf {
-            class,
-            method_booleanConf: env.get_static_method_id(
-                class,
-                "booleanConf",
-                "(Ljava/lang/String;)Z",
-            )?,
-            method_booleanConf_ret: ReturnType::Primitive(Primitive::Boolean),
-            method_intConf: env.get_static_method_id(class, "intConf", "(Ljava/lang/String;)I")?,
-            method_intConf_ret: ReturnType::Primitive(Primitive::Int),
-            method_longConf: env.get_static_method_id(
-                class,
-                "longConf",
-                "(Ljava/lang/String;)J",
-            )?,
-            method_longConf_ret: ReturnType::Primitive(Primitive::Long),
-            method_doubleConf: env.get_static_method_id(
-                class,
-                "doubleConf",
-                "(Ljava/lang/String;)D",
-            )?,
-            method_doubleConf_ret: ReturnType::Primitive(Primitive::Double),
-            method_stringConf: env.get_static_method_id(
-                class,
-                "stringConf",
-                "(Ljava/lang/String;)Ljava/lang/String;",
-            )?,
-            method_stringConf_ret: ReturnType::Object,
-        })
-    }
-}
-
 #[allow(non_snake_case)]
 pub struct AuronRssPartitionWriterBase<'a> {
     pub class: JClass<'a>,
@@ -1193,18 +1184,18 @@ impl<'a> AuronRssPartitionWriterBase<'_> {
 }
 
 #[allow(non_snake_case)]
-pub struct SparkUDFWrapperContext<'a> {
+pub struct SparkAuronUDFWrapperContext<'a> {
     pub class: JClass<'a>,
     pub ctor: JMethodID,
     pub method_eval: JMethodID,
     pub method_eval_ret: ReturnType,
 }
-impl<'a> SparkUDFWrapperContext<'a> {
-    pub const SIG_TYPE: &'static str = "org/apache/spark/sql/auron/SparkUDFWrapperContext";
+impl<'a> SparkAuronUDFWrapperContext<'a> {
+    pub const SIG_TYPE: &'static str = "org/apache/auron/spark/sql/SparkAuronUDFWrapperContext";
 
-    pub fn new(env: &JNIEnv<'a>) -> JniResult<SparkUDFWrapperContext<'a>> {
+    pub fn new(env: &JNIEnv<'a>) -> JniResult<SparkAuronUDFWrapperContext<'a>> {
         let class = get_global_jclass(env, Self::SIG_TYPE)?;
-        Ok(SparkUDFWrapperContext {
+        Ok(SparkAuronUDFWrapperContext {
             class,
             ctor: env.get_method_id(class, "<init>", "(Ljava/nio/ByteBuffer;)V")?,
             method_eval: env.get_method_id(class, "eval", "(JJ)V")?,
@@ -1213,6 +1204,25 @@ impl<'a> SparkUDFWrapperContext<'a> {
     }
 }
 
+#[allow(non_snake_case)]
+pub struct AuronUDFWrapperContext<'a> {
+    pub class: JClass<'a>,
+    pub method_eval: JMethodID,
+    pub method_eval_ret: ReturnType,
+}
+impl<'a> AuronUDFWrapperContext<'a> {
+    pub const SIG_TYPE: &'static str = "org/apache/auron/functions/AuronUDFWrapperContext";
+
+    pub fn new(env: &JNIEnv<'a>) -> JniResult<AuronUDFWrapperContext<'a>> {
+        let class = get_global_jclass(env, Self::SIG_TYPE)?;
+        Ok(AuronUDFWrapperContext {
+            class,
+            method_eval: env.get_method_id(class, "eval", "(JJ)V")?,
+            method_eval_ret: ReturnType::Primitive(Primitive::Void),
+        })
+    }
+}
+
 #[allow(non_snake_case)]
 pub struct SparkUDAFWrapperContext<'a> {
     pub class: JClass<'a>,
@@ -1229,10 +1239,10 @@ pub struct SparkUDAFWrapperContext<'a> {
     pub method_merge_ret: ReturnType,
     pub method_eval: JMethodID,
     pub method_eval_ret: ReturnType,
-    pub method_serializeRows: JMethodID,
-    pub method_serializeRows_ret: ReturnType,
-    pub method_deserializeRows: JMethodID,
-    pub method_deserializeRows_ret: ReturnType,
+    pub method_exportRows: JMethodID,
+    pub method_exportRows_ret: ReturnType,
+    pub method_importRows: JMethodID,
+    pub method_importRows_ret: ReturnType,
     pub method_spill: JMethodID,
     pub method_spill_ret: ReturnType,
     pub method_unspill: JMethodID,
@@ -1282,18 +1292,18 @@ impl<'a> SparkUDAFWrapperContext<'a> {
                 "(Lorg/apache/spark/sql/auron/BufferRowsColumn;[IJ)V",
             )?,
             method_eval_ret: ReturnType::Primitive(Primitive::Void),
-            method_serializeRows: env.get_method_id(
+            method_exportRows: env.get_method_id(
                 class,
-                "serializeRows",
-                "(Lorg/apache/spark/sql/auron/BufferRowsColumn;[I)[B",
+                "exportRows",
+                "(Lorg/apache/spark/sql/auron/BufferRowsColumn;[IJ)V",
             )?,
-            method_serializeRows_ret: ReturnType::Array,
-            method_deserializeRows: env.get_method_id(
+            method_exportRows_ret: ReturnType::Primitive(Primitive::Void),
+            method_importRows: env.get_method_id(
                 class,
-                "deserializeRows",
-                "(Ljava/nio/ByteBuffer;)Lorg/apache/spark/sql/auron/BufferRowsColumn;",
+                "importRows",
+                "(J)Lorg/apache/spark/sql/auron/BufferRowsColumn;",
             )?,
-            method_deserializeRows_ret: ReturnType::Object,
+            method_importRows_ret: ReturnType::Object,
             method_spill: env.get_method_id(
                 class,
                 "spill",
@@ -1387,7 +1397,7 @@ pub struct AuronCallNativeWrapper<'a> {
     pub method_setError_ret: ReturnType,
 }
 impl<'a> AuronCallNativeWrapper<'a> {
-    pub const SIG_TYPE: &'static str = "org/apache/spark/sql/auron/AuronCallNativeWrapper";
+    pub const SIG_TYPE: &'static str = "org/apache/auron/jni/AuronCallNativeWrapper";
 
     pub fn new(env: &JNIEnv<'a>) -> JniResult<AuronCallNativeWrapper<'a>> {
         let class = get_global_jclass(env, Self::SIG_TYPE)?;
@@ -1402,7 +1412,7 @@ impl<'a> AuronCallNativeWrapper<'a> {
             method_getMetrics: env.get_method_id(
                 class,
                 "getMetrics",
-                "()Lorg/apache/spark/sql/auron/MetricNode;",
+                "()Lorg/apache/auron/metric/MetricNode;",
             )?,
             method_getMetrics_ret: ReturnType::Object,
             method_importSchema: env.get_method_id(class, "importSchema", "(J)V")?,
@@ -1434,7 +1444,7 @@ pub struct AuronOnHeapSpillManager<'a> {
     pub method_releaseSpill_ret: ReturnType,
 }
 impl<'a> AuronOnHeapSpillManager<'a> {
-    pub const SIG_TYPE: &'static str = "org/apache/spark/sql/auron/memory/OnHeapSpillManager";
+    pub const SIG_TYPE: &'static str = "org/apache/spark/sql/auron/memory/SparkOnHeapSpillManager";
 
     pub fn new(env: &JNIEnv<'a>) -> JniResult<AuronOnHeapSpillManager<'a>> {
         let class = get_global_jclass(env, Self::SIG_TYPE)?;
@@ -1562,8 +1572,7 @@ pub struct AuronArrowFFIExporter<'a> {
 }
 
 impl<'a> AuronArrowFFIExporter<'a> {
-    pub const SIG_TYPE: &'static str =
-        "org/apache/spark/sql/execution/auron/arrowio/ArrowFFIExporter";
+    pub const SIG_TYPE: &'static str = "org/apache/auron/arrowio/AuronArrowFFIExporter";
 
     pub fn new(env: &JNIEnv<'a>) -> JniResult<AuronArrowFFIExporter<'a>> {
         let class = get_global_jclass(env, Self::SIG_TYPE)?;
@@ -1583,7 +1592,7 @@ pub struct AuronFSDataInputWrapper<'a> {
 }
 
 impl<'a> AuronFSDataInputWrapper<'a> {
-    pub const SIG_TYPE: &'static str = "org/apache/spark/auron/FSDataInputWrapper";
+    pub const SIG_TYPE: &'static str = "org/apache/auron/hadoop/fs/FSDataInputWrapper";
 
     pub fn new(env: &JNIEnv<'a>) -> JniResult<AuronFSDataInputWrapper<'a>> {
         let class = get_global_jclass(env, Self::SIG_TYPE)?;
@@ -1603,7 +1612,7 @@ pub struct AuronFSDataOutputWrapper<'a> {
 }
 
 impl<'a> AuronFSDataOutputWrapper<'a> {
-    pub const SIG_TYPE: &'static str = "org/apache/spark/auron/FSDataOutputWrapper";
+    pub const SIG_TYPE: &'static str = "org/apache/auron/hadoop/fs/FSDataOutputWrapper";
 
     pub fn new(env: &JNIEnv<'a>) -> JniResult<AuronFSDataOutputWrapper<'a>> {
         let class = get_global_jclass(env, Self::SIG_TYPE)?;
