@@ -162,11 +162,15 @@ pub fn cast_impl(
                         // correct orc map entries field name from "keys" to "key", "values" to
                         // "value"
                         if col.is_none() && (origin.eq("key") || origin.eq("value")) {
-                            let adjust = format!("{}s", origin);
+                            let adjust = format!("{origin}s");
                             col = struct_.column_by_name(adjust.as_str());
                         }
                         if col.is_some() {
-                            cast_impl(col.unwrap(), field.data_type(), match_struct_fields)
+                            cast_impl(
+                                col.expect("column missing"),
+                                field.data_type(),
+                                match_struct_fields,
+                            )
                         } else {
                             null_column_name.push(field.name().clone());
                             Ok(new_null_array(field.data_type(), struct_.len()))
@@ -219,6 +223,101 @@ pub fn cast_impl(
         (&DataType::Utf8, DataType::Decimal128(..)) => {
             arrow::compute::kernels::cast::cast(&to_plain_string_array(array), cast_type)?
         }
+        // map to string (spark compatible)
+        (&DataType::Map(..), &DataType::Utf8) => {
+            let map_array = as_map_array(array);
+            let entries = map_array.entries();
+            let keys = entries.column(0);
+            let values = entries.column(1);
+
+            let casted_keys = cast_impl(keys, &DataType::Utf8, match_struct_fields)?;
+            let casted_values = cast_impl(values, &DataType::Utf8, match_struct_fields)?;
+
+            let string_keys = as_string_array(&casted_keys);
+            let string_values = as_string_array(&casted_values);
+
+            let mut builder = StringBuilder::new();
+
+            for row_idx in 0..map_array.len() {
+                if map_array.is_null(row_idx) {
+                    builder.append_null();
+                } else {
+                    let mut row_str = String::from("{");
+                    let start = map_array.value_offsets()[row_idx] as usize;
+                    let end = map_array.value_offsets()[row_idx + 1] as usize;
+
+                    for i in start..end {
+                        if i > start {
+                            row_str.push_str(", ");
+                        }
+
+                        row_str.push_str(string_keys.value(i));
+                        row_str.push_str(" ->");
+
+                        if values.is_null(i) {
+                            row_str.push_str(" null");
+                        } else {
+                            row_str.push(' ');
+                            row_str.push_str(string_values.value(i));
+                        }
+                    }
+
+                    row_str.push('}');
+                    builder.append_value(&row_str);
+                }
+            }
+
+            Arc::new(builder.finish())
+        }
+        // struct to string (spark compatible)
+        (&DataType::Struct(_), &DataType::Utf8) => {
+            let struct_array = as_struct_array(array);
+            let num_columns = struct_array.num_columns();
+
+            let casted_columns: Vec<ArrayRef> = struct_array
+                .columns()
+                .iter()
+                .map(|col| cast_impl(col, &DataType::Utf8, match_struct_fields))
+                .collect::<Result<Vec<_>>>()?;
+
+            let string_columns: Vec<&StringArray> = casted_columns
+                .iter()
+                .map(|col| as_string_array(col))
+                .collect();
+
+            let mut builder = StringBuilder::new();
+
+            for row_idx in 0..struct_array.len() {
+                if struct_array.is_null(row_idx) {
+                    builder.append_null();
+                } else {
+                    let mut row_str = String::from("{");
+
+                    if num_columns > 0 {
+                        if struct_array.column(0).is_null(row_idx) {
+                            row_str.push_str("null");
+                        } else {
+                            row_str.push_str(string_columns[0].value(row_idx));
+                        }
+
+                        for col_idx in 1..num_columns {
+                            row_str.push(',');
+                            if struct_array.column(col_idx).is_null(row_idx) {
+                                row_str.push_str(" null");
+                            } else {
+                                row_str.push(' ');
+                                row_str.push_str(string_columns[col_idx].value(row_idx));
+                            }
+                        }
+                    }
+
+                    row_str.push('}');
+                    builder.append_value(&row_str);
+                }
+            }
+
+            Arc::new(builder.finish())
+        }
         _ => {
             // default cast
             arrow::compute::kernels::cast::cast(array, cast_type)?
@@ -227,7 +326,10 @@ pub fn cast_impl(
 }
 
 fn to_plain_string_array(array: &dyn Array) -> ArrayRef {
-    let array = array.as_any().downcast_ref::<StringArray>().unwrap();
+    let array = array
+        .as_any()
+        .downcast_ref::<StringArray>()
+        .expect("Expected a StringArray");
     let mut converted_values: Vec<Option<String>> = Vec::with_capacity(array.len());
     for v in array.iter() {
         match v {
@@ -252,10 +354,13 @@ fn try_cast_string_array_to_integer(array: &dyn Array, cast_type: &DataType) ->
     macro_rules! cast {
         ($target_type:ident) => {{
             type B = paste::paste! {[<$target_type Builder>]};
-            let array = array.as_any().downcast_ref::<StringArray>().unwrap();
+            let string_array = array
+                .as_any()
+                .downcast_ref::<StringArray>()
+                .expect("Expected a StringArray");
             let mut builder = B::new();
 
-            for v in array.iter() {
+            for v in string_array.iter() {
                 match v {
                     Some(s) => builder.append_option(to_integer(s)),
                     None => builder.append_null(),
@@ -278,7 +383,7 @@ fn try_cast_string_array_to_date(array: &dyn Array) -> Result<ArrayRef> {
     let strings = array.as_string::<i32>();
     let mut converted_values = Vec::with_capacity(strings.len());
     for s in strings {
-        converted_values.push(s.and_then(|s| to_date(s)));
+        converted_values.push(s.and_then(to_date));
     }
     Ok(Arc::new(Date32Array::from(converted_values)))
 }
@@ -305,7 +410,7 @@ fn to_integer<T: Bounded + FromPrimitive + Integer + Signed + Copy>(input: &str)
     }
 
     let separator = b'.';
-    let radix = T::from_usize(10).unwrap();
+    let radix = T::from_usize(10).expect("from_usize(10) failed");
     let stop_value = T::min_value() / radix;
     let mut result = T::zero();
 
@@ -333,7 +438,7 @@ fn to_integer<T: Bounded + FromPrimitive + Integer + Signed + Copy>(input: &str)
             return None;
         }
 
-        result = result * radix - T::from_u8(digit).unwrap();
+        result = result * radix - T::from_u8(digit).expect("digit must be in 0..=9");
         // Since the previous result is less than or equal to stopValue(Long.MIN_VALUE /
         // radix), we can just use `result > 0` to check overflow. If result
         // overflows, we should stop.
@@ -397,7 +502,7 @@ fn to_date(s: &str) -> Option<i32> {
             i += 1;
         } else {
             let parsed_value = (b - b'0') as i32;
-            if parsed_value < 0 || parsed_value > 9 {
+            if !(0..=9).contains(&parsed_value) {
                 return None;
             } else {
                 current_segment_value = current_segment_value * 10 + parsed_value;
@@ -425,23 +530,27 @@ fn to_date(s: &str) -> Option<i32> {
 
 #[cfg(test)]
 mod test {
-    use datafusion::common::cast::{as_decimal128_array, as_float64_array, as_int32_array};
+    use datafusion::common::{
+        Result,
+        cast::{as_decimal128_array, as_float64_array, as_int32_array},
+    };
 
     use super::*;
 
     #[test]
-    fn test_boolean_to_string() {
+    fn test_boolean_to_string() -> Result<()> {
         let bool_array: ArrayRef =
             Arc::new(BooleanArray::from_iter(vec![None, Some(true), Some(false)]));
-        let casted = cast(&bool_array, &DataType::Utf8).unwrap();
+        let casted = cast(&bool_array, &DataType::Utf8)?;
         assert_eq!(
             as_string_array(&casted),
             &StringArray::from_iter(vec![None, Some("true"), Some("false")])
         );
+        Ok(())
     }
 
     #[test]
-    fn test_float_to_int() {
+    fn test_float_to_int() -> Result<()> {
         let f64_array: ArrayRef = Arc::new(Float64Array::from_iter(vec![
             None,
             Some(123.456),
@@ -452,9 +561,9 @@ mod test {
             Some(f64::NEG_INFINITY),
             Some(f64::NAN),
         ]));
-        let casted = cast(&f64_array, &DataType::Int32).unwrap();
+        let casted = cast(&f64_array, &DataType::Int32)?;
         assert_eq!(
-            as_int32_array(&casted).unwrap(),
+            as_int32_array(&casted)?,
             &Int32Array::from_iter(vec![
                 None,
                 Some(123),
@@ -466,10 +575,11 @@ mod test {
                 Some(0),
             ])
         );
+        Ok(())
     }
 
     #[test]
-    fn test_int_to_float() {
+    fn test_int_to_float() -> Result<()> {
         let i32_array: ArrayRef = Arc::new(Int32Array::from_iter(vec![
             None,
             Some(123),
@@ -477,9 +587,9 @@ mod test {
             Some(i32::MAX),
             Some(i32::MIN),
         ]));
-        let casted = cast(&i32_array, &DataType::Float64).unwrap();
+        let casted = cast(&i32_array, &DataType::Float64)?;
         assert_eq!(
-            as_float64_array(&casted).unwrap(),
+            as_float64_array(&casted)?,
             &Float64Array::from_iter(vec![
                 None,
                 Some(123.0),
@@ -488,10 +598,11 @@ mod test {
                 Some(i32::MIN as f64),
             ])
         );
+        Ok(())
     }
 
     #[test]
-    fn test_int_to_decimal() {
+    fn test_int_to_decimal() -> Result<()> {
         let i32_array: ArrayRef = Arc::new(Int32Array::from_iter(vec![
             None,
             Some(123),
@@ -499,9 +610,9 @@ mod test {
             Some(i32::MAX),
             Some(i32::MIN),
         ]));
-        let casted = cast(&i32_array, &DataType::Decimal128(38, 18)).unwrap();
+        let casted = cast(&i32_array, &DataType::Decimal128(38, 18))?;
         assert_eq!(
-            as_decimal128_array(&casted).unwrap(),
+            as_decimal128_array(&casted)?,
             &Decimal128Array::from_iter(vec![
                 None,
                 Some(123000000000000000000),
@@ -509,13 +620,13 @@ mod test {
                 Some(i32::MAX as i128 * 1000000000000000000),
                 Some(i32::MIN as i128 * 1000000000000000000),
             ])
-            .with_precision_and_scale(38, 18)
-            .unwrap()
+            .with_precision_and_scale(38, 18)?
         );
+        Ok(())
     }
 
     #[test]
-    fn test_string_to_decimal() {
+    fn test_string_to_decimal() -> Result<()> {
         let string_array: ArrayRef = Arc::new(StringArray::from_iter(vec![
             None,
             Some("1e-8"),
@@ -527,9 +638,9 @@ mod test {
             Some("123456789012345.678901234567890"),
             Some("-123456789012345.678901234567890"),
         ]));
-        let casted = cast(&string_array, &DataType::Decimal128(38, 18)).unwrap();
+        let casted = cast(&string_array, &DataType::Decimal128(38, 18))?;
         assert_eq!(
-            as_decimal128_array(&casted).unwrap(),
+            as_decimal128_array(&casted)?,
             &Decimal128Array::from_iter(vec![
                 None,
                 Some(10000000000),
@@ -541,13 +652,13 @@ mod test {
                 Some(123456789012345678901234567890000i128),
                 Some(-123456789012345678901234567890000i128),
             ])
-            .with_precision_and_scale(38, 18)
-            .unwrap()
+            .with_precision_and_scale(38, 18)?
         );
+        Ok(())
     }
 
     #[test]
-    fn test_decimal_to_string() {
+    fn test_decimal_to_string() -> Result<()> {
         let decimal_array: ArrayRef = Arc::new(
             Decimal128Array::from_iter(vec![
                 None,
@@ -557,12 +668,14 @@ mod test {
                 Some(i32::MAX as i128 * 1000000000000000000),
                 Some(i32::MIN as i128 * 1000000000000000000),
             ])
-            .with_precision_and_scale(38, 18)
-            .unwrap(),
+            .with_precision_and_scale(38, 18)?,
         );
-        let casted = cast(&decimal_array, &DataType::Utf8).unwrap();
+        let casted = cast(&decimal_array, &DataType::Utf8)?;
         assert_eq!(
-            casted.as_any().downcast_ref::<StringArray>().unwrap(),
+            casted
+                .as_any()
+                .downcast_ref::<StringArray>()
+                .expect("Expected a StringArray"),
             &StringArray::from_iter(vec![
                 None,
                 Some("123.000000000000000000"),
@@ -572,10 +685,11 @@ mod test {
                 Some("-2147483648.000000000000000000"),
             ])
         );
+        Ok(())
     }
 
     #[test]
-    fn test_string_to_bigint() {
+    fn test_string_to_bigint() -> Result<()> {
         let string_array: ArrayRef = Arc::new(StringArray::from_iter(vec![
             None,
             Some("123"),
@@ -585,9 +699,12 @@ mod test {
             Some("-123456789012345"),
             Some("999999999999999999999999999999999"),
         ]));
-        let casted = cast(&string_array, &DataType::Int64).unwrap();
+        let casted = cast(&string_array, &DataType::Int64)?;
         assert_eq!(
-            casted.as_any().downcast_ref::<Int64Array>().unwrap(),
+            casted
+                .as_any()
+                .downcast_ref::<Int64Array>()
+                .expect("Expected a Int64Array"),
             &Int64Array::from_iter(vec![
                 None,
                 Some(123),
@@ -598,10 +715,11 @@ mod test {
                 None,
             ])
         );
+        Ok(())
     }
 
     #[test]
-    fn test_string_to_date() {
+    fn test_string_to_date() -> Result<()> {
         let string_array: ArrayRef = Arc::new(StringArray::from_iter(vec![
             None,
             Some("2001-02-03"),
@@ -614,11 +732,9 @@ mod test {
             Some("9999-99"),
             Some("99999-01"),
         ]));
-        let casted = cast(&string_array, &DataType::Date32).unwrap();
+        let casted = cast(&string_array, &DataType::Date32)?;
         assert_eq!(
-            arrow::compute::cast(&casted, &DataType::Utf8)
-                .unwrap()
-                .as_string(),
+            arrow::compute::cast(&casted, &DataType::Utf8)?.as_string(),
             &StringArray::from_iter(vec![
                 None,
                 Some("2001-02-03"),
@@ -632,5 +748,299 @@ mod test {
                 None,
             ])
         );
+        Ok(())
+    }
+
+    #[test]
+    fn test_struct_to_string() -> Result<()> {
+        // Create a struct array with fields: (int32, string, boolean)
+        let int_array = Int32Array::from(vec![Some(1), Some(2), None, Some(4), None]);
+        let string_array = StringArray::from(vec![Some("a"), None, Some("c"), Some("d"), None]);
+        let bool_array = BooleanArray::from(vec![Some(true), Some(false), Some(true), None, None]);
+
+        let struct_array: ArrayRef = Arc::new(StructArray::from(vec![
+            (
+                Arc::new(Field::new("f1", DataType::Int32, true)),
+                Arc::new(int_array) as ArrayRef,
+            ),
+            (
+                Arc::new(Field::new("f2", DataType::Utf8, true)),
+                Arc::new(string_array) as ArrayRef,
+            ),
+            (
+                Arc::new(Field::new("f3", DataType::Boolean, true)),
+                Arc::new(bool_array) as ArrayRef,
+            ),
+        ]));
+
+        let casted = cast(&struct_array, &DataType::Utf8)?;
+        assert_eq!(
+            as_string_array(&casted),
+            &StringArray::from_iter(vec![
+                Some("{1, a, true}"),
+                Some("{2, null, false}"),
+                Some("{null, c, true}"),
+                Some("{4, d, null}"),
+                Some("{null, null, null}"),
+            ])
+        );
+        Ok(())
+    }
+
+    #[test]
+    fn test_struct_to_string_with_null_struct() -> Result<()> {
+        // Create a struct array where some rows are entirely null
+        let int_array = Int32Array::from(vec![Some(1), Some(2), Some(3)]);
+        let string_array = StringArray::from(vec![Some("a"), Some("b"), Some("c")]);
+
+        let fields = Fields::from(vec![
+            Field::new("f1", DataType::Int32, true),
+            Field::new("f2", DataType::Utf8, true),
+        ]);
+
+        // Set null at index 1
+        let nulls = arrow::buffer::NullBuffer::from(vec![true, false, true]);
+        let struct_array_with_nulls: ArrayRef = Arc::new(StructArray::new(
+            fields,
+            vec![
+                Arc::new(int_array) as ArrayRef,
+                Arc::new(string_array) as ArrayRef,
+            ],
+            Some(nulls),
+        ));
+
+        let casted = cast(&struct_array_with_nulls, &DataType::Utf8)?;
+        assert_eq!(
+            as_string_array(&casted),
+            &StringArray::from_iter(vec![Some("{1, a}"), None, Some("{3, c}"),])
+        );
+        Ok(())
+    }
+
+    #[test]
+    fn test_empty_struct_to_string() -> Result<()> {
+        // Create a struct array with zero fields but 2 rows
+        let struct_array: ArrayRef = Arc::new(StructArray::new_empty_fields(2, None));
+
+        let casted = cast(&struct_array, &DataType::Utf8)?;
+        assert_eq!(
+            as_string_array(&casted),
+            &StringArray::from_iter(vec![Some("{}"), Some("{}")])
+        );
+        Ok(())
+    }
+
+    #[test]
+    fn test_nested_struct_to_string() -> Result<()> {
+        // Create a nested struct: struct<int, struct<string, bool>>
+        let inner_string = StringArray::from(vec![Some("x"), Some("y")]);
+        let inner_bool = BooleanArray::from(vec![Some(true), None]);
+
+        let inner_struct: ArrayRef = Arc::new(StructArray::from(vec![
+            (
+                Arc::new(Field::new("s1", DataType::Utf8, true)),
+                Arc::new(inner_string) as ArrayRef,
+            ),
+            (
+                Arc::new(Field::new("s2", DataType::Boolean, true)),
+                Arc::new(inner_bool) as ArrayRef,
+            ),
+        ]));
+
+        let outer_int = Int32Array::from(vec![Some(100), Some(200)]);
+
+        let outer_struct: ArrayRef = Arc::new(StructArray::from(vec![
+            (
+                Arc::new(Field::new("f1", DataType::Int32, true)),
+                Arc::new(outer_int) as ArrayRef,
+            ),
+            (
+                Arc::new(Field::new("f2", inner_struct.data_type().clone(), true)),
+                inner_struct,
+            ),
+        ]));
+
+        let casted = cast(&outer_struct, &DataType::Utf8)?;
+        assert_eq!(
+            as_string_array(&casted),
+            &StringArray::from_iter(vec![Some("{100, {x, true}}"), Some("{200, {y, null}}"),])
+        );
+        Ok(())
+    }
+
+    #[test]
+    fn test_map_to_string() -> Result<()> {
+        // Create a map array: Map<Int32, String>
+        let key_field = Arc::new(Field::new("key", DataType::Int32, false));
+        let value_field = Arc::new(Field::new("value", DataType::Utf8, true));
+        let entries_field = Arc::new(Field::new(
+            "entries",
+            DataType::Struct(Fields::from(vec![
+                key_field.as_ref().clone(),
+                value_field.as_ref().clone(),
+            ])),
+            false,
+        ));
+
+        let keys = Int32Array::from(vec![1, 2, 3, 4, 5]);
+        let values = StringArray::from(vec![Some("a"), Some("b"), None, Some("d"), Some("e")]);
+
+        let entries = StructArray::from(vec![
+            (key_field.clone(), Arc::new(keys) as ArrayRef),
+            (value_field.clone(), Arc::new(values) as ArrayRef),
+        ]);
+
+        let offsets = arrow::buffer::OffsetBuffer::new(vec![0i32, 2, 3, 5].into());
+        let map_array: ArrayRef = Arc::new(MapArray::new(
+            entries_field.clone(),
+            offsets,
+            entries,
+            None,
+            false,
+        ));
+
+        let casted = cast(&map_array, &DataType::Utf8)?;
+        assert_eq!(
+            as_string_array(&casted),
+            &StringArray::from_iter(vec![
+                Some("{1 -> a, 2 -> b}"),
+                Some("{3 -> null}"),
+                Some("{4 -> d, 5 -> e}"),
+            ])
+        );
+        Ok(())
+    }
+
+    #[test]
+    fn test_map_to_string_with_null_map() -> Result<()> {
+        // Create a map array with null rows
+        let key_field = Arc::new(Field::new("key", DataType::Int32, false));
+        let value_field = Arc::new(Field::new("value", DataType::Utf8, true));
+        let entries_field = Arc::new(Field::new(
+            "entries",
+            DataType::Struct(Fields::from(vec![
+                key_field.as_ref().clone(),
+                value_field.as_ref().clone(),
+            ])),
+            false,
+        ));
+
+        let keys = Int32Array::from(vec![1, 2, 3]);
+        let values = StringArray::from(vec![Some("a"), Some("b"), Some("c")]);
+
+        let entries = StructArray::from(vec![
+            (key_field.clone(), Arc::new(keys) as ArrayRef),
+            (value_field.clone(), Arc::new(values) as ArrayRef),
+        ]);
+
+        let offsets = arrow::buffer::OffsetBuffer::new(vec![0i32, 1, 2, 3].into());
+        let nulls = arrow::buffer::NullBuffer::from(vec![true, false, true]);
+        let map_array: ArrayRef = Arc::new(MapArray::new(
+            entries_field.clone(),
+            offsets,
+            entries,
+            Some(nulls),
+            false,
+        ));
+
+        let casted = cast(&map_array, &DataType::Utf8)?;
+        assert_eq!(
+            as_string_array(&casted),
+            &StringArray::from_iter(vec![Some("{1 -> a}"), None, Some("{3 -> c}"),])
+        );
+        Ok(())
+    }
+
+    #[test]
+    fn test_empty_map_to_string() -> Result<()> {
+        // Create an empty map array
+        let key_field = Arc::new(Field::new("key", DataType::Int32, false));
+        let value_field = Arc::new(Field::new("value", DataType::Utf8, true));
+        let entries_field = Arc::new(Field::new(
+            "entries",
+            DataType::Struct(Fields::from(vec![
+                key_field.as_ref().clone(),
+                value_field.as_ref().clone(),
+            ])),
+            false,
+        ));
+
+        let keys = Int32Array::from(vec![] as Vec<i32>);
+        let values = StringArray::from(vec![] as Vec<Option<&str>>);
+
+        let entries = StructArray::from(vec![
+            (key_field.clone(), Arc::new(keys) as ArrayRef),
+            (value_field.clone(), Arc::new(values) as ArrayRef),
+        ]);
+
+        // Two rows, both empty maps
+        let offsets = arrow::buffer::OffsetBuffer::new(vec![0i32, 0, 0].into());
+        let map_array: ArrayRef = Arc::new(MapArray::new(
+            entries_field.clone(),
+            offsets,
+            entries,
+            None,
+            false,
+        ));
+
+        let casted = cast(&map_array, &DataType::Utf8)?;
+        assert_eq!(
+            as_string_array(&casted),
+            &StringArray::from_iter(vec![Some("{}"), Some("{}")])
+        );
+        Ok(())
+    }
+
+    #[test]
+    fn test_nested_map_to_string() -> Result<()> {
+        // Create a map with struct values: Map<Int32, Struct<String, Boolean>>
+        let key_field = Arc::new(Field::new("key", DataType::Int32, false));
+
+        let inner_string_field = Arc::new(Field::new("s1", DataType::Utf8, true));
+        let inner_bool_field = Arc::new(Field::new("s2", DataType::Boolean, true));
+        let inner_struct_type = DataType::Struct(Fields::from(vec![
+            inner_string_field.as_ref().clone(),
+            inner_bool_field.as_ref().clone(),
+        ]));
+
+        let value_field = Arc::new(Field::new("value", inner_struct_type.clone(), true));
+        let entries_field = Arc::new(Field::new(
+            "entries",
+            DataType::Struct(Fields::from(vec![
+                key_field.as_ref().clone(),
+                value_field.as_ref().clone(),
+            ])),
+            false,
+        ));
+
+        let keys = Int32Array::from(vec![1, 2]);
+        let inner_strings = StringArray::from(vec![Some("x"), Some("y")]);
+        let inner_bools = BooleanArray::from(vec![Some(true), None]);
+        let inner_struct: ArrayRef = Arc::new(StructArray::from(vec![
+            (inner_string_field, Arc::new(inner_strings) as ArrayRef),
+            (inner_bool_field, Arc::new(inner_bools) as ArrayRef),
+        ]));
+
+        let entries = StructArray::from(vec![
+            (key_field.clone(), Arc::new(keys) as ArrayRef),
+            (value_field.clone(), inner_struct),
+        ]);
+
+        // One row with 2 entries
+        let offsets = arrow::buffer::OffsetBuffer::new(vec![0i32, 2].into());
+        let map_array: ArrayRef = Arc::new(MapArray::new(
+            entries_field.clone(),
+            offsets,
+            entries,
+            None,
+            false,
+        ));
+
+        let casted = cast(&map_array, &DataType::Utf8)?;
+        assert_eq!(
+            as_string_array(&casted),
+            &StringArray::from_iter(vec![Some("{1 -> {x, true}, 2 -> {y, null}}")])
+        );
+        Ok(())
     }
 }
