@@ -56,6 +56,7 @@ pub fn create_hashes<T: num::PrimInt>(
     hash_buffer
 }
 
+#[allow(clippy::panic)] // Temporarily allow panic to refactor to Result later
 #[inline]
 fn hash_array<T: num::PrimInt>(
     array: &ArrayRef,
@@ -66,7 +67,10 @@ fn hash_array<T: num::PrimInt>(
 
     macro_rules! hash_array {
         ($array_type:ident, $column:ident, $hashes:ident, $h:expr) => {
-            let array = $column.as_any().downcast_ref::<$array_type>().unwrap();
+            let array = $column
+                .as_any()
+                .downcast_ref::<$array_type>()
+                .expect("downcast to expected array type failed");
             if array.null_count() == 0 {
                 for (i, hash) in $hashes.iter_mut().enumerate() {
                     *hash = $h(&array.value(i).as_ref(), *hash);
@@ -83,7 +87,10 @@ fn hash_array<T: num::PrimInt>(
 
     macro_rules! hash_array_primitive {
         ($array_type:ident, $column:ident, $ty:ident, $hashes:ident, $h:expr) => {
-            let array = $column.as_any().downcast_ref::<$array_type>().unwrap();
+            let array = $column
+                .as_any()
+                .downcast_ref::<$array_type>()
+                .expect("downcast to expected array type failed");
             let values = array.values();
 
             if array.null_count() == 0 {
@@ -102,7 +109,10 @@ fn hash_array<T: num::PrimInt>(
 
     macro_rules! hash_array_decimal {
         ($array_type:ident, $column:ident, $hashes:ident, $h:expr) => {
-            let array = $column.as_any().downcast_ref::<$array_type>().unwrap();
+            let array = $column
+                .as_any()
+                .downcast_ref::<$array_type>()
+                .expect("downcast to expected array type failed");
 
             if array.null_count() == 0 {
                 for (i, hash) in $hashes.iter_mut().enumerate() {
@@ -121,7 +131,10 @@ fn hash_array<T: num::PrimInt>(
     match array.data_type() {
         DataType::Null => {}
         DataType::Boolean => {
-            let array = array.as_any().downcast_ref::<BooleanArray>().unwrap();
+            let array = array
+                .as_any()
+                .downcast_ref::<BooleanArray>()
+                .expect("Expected a BooleanArray");
             if array.null_count() == 0 {
                 for (i, hash) in hashes_buffer.iter_mut().enumerate() {
                     *hash = h(
@@ -217,7 +230,10 @@ fn create_hashes_dictionary<K: ArrowDictionaryKeyType, T: num::PrimInt>(
     hashes_buffer: &mut [T],
     h: impl Fn(&[u8], T) -> T + Copy,
 ) {
-    let dict_array = array.as_any().downcast_ref::<DictionaryArray<K>>().unwrap();
+    let dict_array = array
+        .as_any()
+        .downcast_ref::<DictionaryArray<K>>()
+        .expect("Expected a DictionaryArray");
 
     // Hash each dictionary value once, and then use that computed
     // hash for each key value to avoid a potentially expensive
@@ -230,6 +246,7 @@ fn create_hashes_dictionary<K: ArrowDictionaryKeyType, T: num::PrimInt>(
     }
 }
 
+#[allow(clippy::panic)] // Temporarily allow panic to refactor to Result later
 fn hash_one<T: num::PrimInt>(
     col: &ArrayRef,
     idx: usize,
@@ -238,7 +255,10 @@ fn hash_one<T: num::PrimInt>(
 ) {
     macro_rules! hash_one_primitive {
         ($array_type:ident, $column:ident, $ty:ident, $hash:ident, $idx:ident, $h:expr) => {
-            let array = $column.as_any().downcast_ref::<$array_type>().unwrap();
+            let array = $column
+                .as_any()
+                .downcast_ref::<$array_type>()
+                .expect("downcast to expected array type failed");
             *$hash = $h(
                 (array.value($idx as usize) as $ty).to_le_bytes().as_ref(),
                 *$hash,
@@ -248,14 +268,20 @@ fn hash_one<T: num::PrimInt>(
 
     macro_rules! hash_one_binary {
         ($array_type:ident, $column:ident, $hash:ident, $idx:ident, $h:expr) => {
-            let array = $column.as_any().downcast_ref::<$array_type>().unwrap();
+            let array = $column
+                .as_any()
+                .downcast_ref::<$array_type>()
+                .expect("downcast to expected array type failed");
             *$hash = $h(&array.value($idx as usize).as_ref(), *$hash);
         };
     }
 
     macro_rules! hash_one_decimal {
         ($array_type:ident, $column:ident, $hash:ident, $idx:ident, $h:expr) => {
-            let array = $column.as_any().downcast_ref::<$array_type>().unwrap();
+            let array = $column
+                .as_any()
+                .downcast_ref::<$array_type>()
+                .expect("downcast to expected array type failed");
             *$hash = $h(array.value($idx as usize).to_le_bytes().as_ref(), *$hash);
         };
     }
@@ -264,7 +290,10 @@ fn hash_one<T: num::PrimInt>(
         match col.data_type() {
             DataType::Null => {}
             DataType::Boolean => {
-                let array = col.as_any().downcast_ref::<BooleanArray>().unwrap();
+                let array = col
+                    .as_any()
+                    .downcast_ref::<BooleanArray>()
+                    .expect("Expected a BooleanArray");
                 *hash = h(
                     (if array.value(idx) { 1u32 } else { 0u32 })
                         .to_le_bytes()
@@ -324,14 +353,20 @@ fn hash_one<T: num::PrimInt>(
                 hash_one_decimal!(Decimal128Array, col, hash, idx, h);
             }
             DataType::List(..) => {
-                let list_array = col.as_any().downcast_ref::<ListArray>().unwrap();
+                let list_array = col
+                    .as_any()
+                    .downcast_ref::<ListArray>()
+                    .expect("Expected a ListArray");
                 let value_array = list_array.value(idx);
                 for i in 0..value_array.len() {
                     hash_one(&value_array, i, hash, h);
                 }
             }
             DataType::Map(..) => {
-                let map_array = col.as_any().downcast_ref::<MapArray>().unwrap();
+                let map_array = col
+                    .as_any()
+                    .downcast_ref::<MapArray>()
+                    .expect("Expected a MapArray");
                 let kv_array = map_array.value(idx);
                 let key_array = kv_array.column(0);
                 let value_array = kv_array.column(1);
@@ -341,7 +376,10 @@ fn hash_one<T: num::PrimInt>(
                 }
             }
             DataType::Struct(_) => {
-                let struct_array = col.as_any().downcast_ref::<StructArray>().unwrap();
+                let struct_array = col
+                    .as_any()
+                    .downcast_ref::<StructArray>()
+                    .expect("Expected a StructArray");
                 for col in struct_array.columns() {
                     hash_one(col, idx, hash, h);
                 }
@@ -353,7 +391,7 @@ fn hash_one<T: num::PrimInt>(
 
 #[cfg(test)]
 mod tests {
-    use std::sync::Arc;
+    use std::{error::Error, sync::Arc};
 
     use arrow::{
         array::{
@@ -368,7 +406,7 @@ mod tests {
 
     #[test]
     fn test_list() {
-        let mut hashes_buffer = vec![42; 4];
+        let mut hashes_buffer = [42; 4];
         for hash in hashes_buffer.iter_mut() {
             *hash = spark_compatible_murmur3_hash(5_i32.to_le_bytes(), *hash);
         }
@@ -482,24 +520,22 @@ mod tests {
     }
 
     #[test]
-    fn test_list_array() {
+    fn test_list_array() -> Result<(), Box<dyn Error>> {
         // Create inner array data: [1, 2, 3, 4, 5, 6]
         let value_data = ArrayData::builder(DataType::Int32)
             .len(6)
             .add_buffer(Buffer::from_slice_ref(
-                &[1i32, 2, 3, 4, 5, 6].to_byte_slice(),
+                [1i32, 2, 3, 4, 5, 6].to_byte_slice(),
             ))
-            .build()
-            .unwrap();
+            .build()?;
 
         // Create offset array to define list boundaries: [[1, 2], [3, 4, 5], [6]]
         let list_data_type = DataType::new_list(DataType::Int32, false);
         let list_data = ArrayData::builder(list_data_type)
             .len(3)
-            .add_buffer(Buffer::from_slice_ref(&[0i32, 2, 5, 6].to_byte_slice()))
+            .add_buffer(Buffer::from_slice_ref([0i32, 2, 5, 6].to_byte_slice()))
             .add_child_data(value_data)
-            .build()
-            .unwrap();
+            .build()?;
 
         let list_array = ListArray::from(list_data);
         let array_ref = Arc::new(list_array) as ArrayRef;
@@ -507,30 +543,29 @@ mod tests {
         // Test Murmur3 hash
         let hashes = create_murmur3_hashes(3, &[array_ref.clone()], 42);
         assert_eq!(hashes, vec![-222940379, -374492525, -331964951]);
+        Ok(())
     }
 
     #[test]
-    fn test_map_array() {
+    fn test_map_array() -> Result<(), Box<dyn Error>> {
         // Construct key and values
         let key_data = ArrayData::builder(DataType::Int32)
             .len(8)
             .add_buffer(Buffer::from_slice_ref(
-                &[0, 1, 2, 3, 4, 5, 6, 7].to_byte_slice(),
+                [0, 1, 2, 3, 4, 5, 6, 7].to_byte_slice(),
             ))
-            .build()
-            .unwrap();
+            .build()?;
         let value_data = ArrayData::builder(DataType::UInt32)
             .len(8)
             .add_buffer(Buffer::from_slice_ref(
-                &[0u32, 10, 20, 0, 40, 0, 60, 70].to_byte_slice(),
+                [0u32, 10, 20, 0, 40, 0, 60, 70].to_byte_slice(),
             ))
             .null_bit_buffer(Some(Buffer::from(&[0b11010110])))
-            .build()
-            .unwrap();
+            .build()?;
 
         // Construct a buffer for value offsets, for the nested array:
         //  [[0, 1, 2], [3, 4, 5], [6, 7]]
-        let entry_offsets = Buffer::from_slice_ref(&[0, 3, 6, 8].to_byte_slice());
+        let entry_offsets = Buffer::from_slice_ref([0, 3, 6, 8].to_byte_slice());
 
         let keys_field = Arc::new(Field::new("keys", DataType::Int32, false));
         let values_field = Arc::new(Field::new("values", DataType::UInt32, true));
@@ -552,8 +587,7 @@ mod tests {
             .len(3)
             .add_buffer(entry_offsets)
             .add_child_data(entry_struct.into_data())
-            .build()
-            .unwrap();
+            .build()?;
         let map_array = MapArray::from(map_data);
 
         assert_eq!(&value_data, &map_array.values().to_data());
@@ -579,7 +613,7 @@ mod tests {
             unsafe { map_array.value_unchecked(0) }
                 .as_any()
                 .downcast_ref::<StructArray>()
-                .unwrap()
+                .expect("Expected a StructArray")
         );
         for i in 0..3 {
             assert!(map_array.is_valid(i));
@@ -592,8 +626,7 @@ mod tests {
             .offset(1)
             .add_buffer(map_array.to_data().buffers()[0].clone())
             .add_child_data(map_array.to_data().child_data()[0].clone())
-            .build()
-            .unwrap();
+            .build()?;
         let map_array = MapArray::from(map_data);
 
         assert_eq!(&value_data, &map_array.values().to_data());
@@ -613,14 +646,15 @@ mod tests {
                 .value(0)
                 .as_any()
                 .downcast_ref::<StructArray>()
-                .unwrap()
+                .expect("Expected a StructArray")
         );
         assert_eq!(
             &struct_array,
             unsafe { map_array.value_unchecked(0) }
                 .as_any()
                 .downcast_ref::<StructArray>()
-                .unwrap()
+                .expect("Expected a StructArray")
         );
+        Ok(())
     }
 }
