@@ -23,6 +23,8 @@ pub mod get_indexed_field;
 pub mod get_map_value;
 pub mod named_struct;
 pub mod row_num;
+pub mod spark_monotonically_increasing_id;
+pub mod spark_partition_id;
 pub mod spark_scalar_subquery_wrapper;
 pub mod spark_udf_wrapper;
 pub mod string_contains;
@@ -31,10 +33,12 @@ pub mod string_starts_with;
 
 fn down_cast_any_ref(any: &dyn Any) -> &dyn Any {
     if any.is::<PhysicalExprRef>() {
-        any.downcast_ref::<PhysicalExprRef>().unwrap().as_any()
+        any.downcast_ref::<PhysicalExprRef>()
+            .expect("Expected a PhysicalExpr")
+            .as_any()
     } else if any.is::<Box<dyn PhysicalExpr>>() {
         any.downcast_ref::<Box<dyn PhysicalExpr>>()
-            .unwrap()
+            .expect("Expected a PhysicalExpr")
             .as_any()
     } else {
         any
