@@ -38,6 +38,9 @@ pub fn string_lower(args: &[ColumnarValue]) -> Result<ColumnarValue> {
         ColumnarValue::Scalar(ScalarValue::Utf8(Some(str))) => Ok(ColumnarValue::Scalar(
             ScalarValue::Utf8(Some(str.to_lowercase())),
         )),
+        ColumnarValue::Scalar(ScalarValue::Utf8(None)) => {
+            Ok(ColumnarValue::Scalar(ScalarValue::Utf8(None)))
+        }
         _ => df_execution_err!("string_lower only supports literal utf8"),
     }
 }
@@ -52,7 +55,10 @@ pub fn string_upper(args: &[ColumnarValue]) -> Result<ColumnarValue> {
         ColumnarValue::Scalar(ScalarValue::Utf8(Some(str))) => Ok(ColumnarValue::Scalar(
             ScalarValue::Utf8(Some(str.to_uppercase())),
         )),
-        _ => df_execution_err!("string_lower only supports literal utf8"),
+        ColumnarValue::Scalar(ScalarValue::Utf8(None)) => {
+            Ok(ColumnarValue::Scalar(ScalarValue::Utf8(None)))
+        }
+        _ => df_execution_err!("string_upper only supports literal utf8"),
     }
 }
 
@@ -143,7 +149,7 @@ pub fn string_concat(args: &[ColumnarValue]) -> Result<ColumnarValue> {
                         }
                         ColumnarValue::Array(v) => {
                             if v.is_valid(index) {
-                                let v = as_string_array(v).unwrap();
+                                let v = as_string_array(v).expect("Expected a StringArray");
                                 owned_string.push_str(v.value(index));
                             } else {
                                 is_not_null = false;
@@ -161,16 +167,15 @@ pub fn string_concat(args: &[ColumnarValue]) -> Result<ColumnarValue> {
     } else {
         // short avenue with only scalars
         // returns null if args contains null
-        let is_not_null = args.iter().all(|arg| match arg {
-            ColumnarValue::Scalar(scalar) if scalar.is_null() => false,
-            _ => true,
-        });
+        let is_not_null = args
+            .iter()
+            .all(|arg| !matches!(arg, ColumnarValue::Scalar(scalar) if scalar.is_null()));
         if !is_not_null {
             return Ok(ColumnarValue::Scalar(ScalarValue::Utf8(None)));
         }
 
         // concat
-        let initial = Some("".to_string());
+        let initial = Some(String::new());
         let result = args.iter().fold(initial, |mut acc, rhs| {
             if let Some(ref mut inner) = acc {
                 match rhs {
@@ -218,11 +223,11 @@ pub fn string_concat_ws(args: &[ColumnarValue]) -> Result<ColumnarValue> {
                     }
                 }
                 ColumnarValue::Scalar(scalar) => {
-                    if let ScalarValue::Utf8(s) = scalar {
-                        match s {
-                            Some(s) => return Ok(Arg::Literal(&s)),
-                            None => return Ok(Arg::Ignore),
-                        }
+                    if scalar.is_null() {
+                        return Ok(Arg::Ignore);
+                    }
+                    if let ScalarValue::Utf8(Some(s)) = scalar {
+                        return Ok(Arg::Literal(s));
                     }
                     if let ScalarValue::List(l) = scalar
                         && l.data_type() == &DataType::Utf8
@@ -293,7 +298,8 @@ pub fn string_concat_ws(args: &[ColumnarValue]) -> Result<ColumnarValue> {
                     }
                     Arg::List(list) => {
                         if list.is_valid(i) {
-                            let strings = as_string_array(list.values()).unwrap();
+                            let strings =
+                                as_string_array(list.values()).expect("Expected a StringArray");
                             let offsets = list.value_offsets();
                             let l = offsets[i] as usize;
                             let r = offsets[i + 1] as usize;
@@ -324,6 +330,7 @@ mod test {
         },
         physical_plan::ColumnarValue,
     };
+    use datafusion_ext_commons::df_execution_err;
 
     use crate::spark_strings::{
         string_concat, string_concat_ws, string_lower, string_repeat, string_space, string_split,
@@ -370,13 +377,31 @@ mod test {
         Ok(())
     }
 
+    #[test]
+    fn test_string_lower_null_scalar() -> Result<()> {
+        let r = string_lower(&vec![ColumnarValue::Scalar(ScalarValue::Utf8(None))])?;
+        match r {
+            ColumnarValue::Scalar(ScalarValue::Utf8(None)) => Ok(()),
+            other => df_execution_err!("Expected null Utf8 scalar, got: {:?}", other),
+        }
+    }
+
+    #[test]
+    fn test_string_upper_null_scalar() -> Result<()> {
+        let r = string_upper(&vec![ColumnarValue::Scalar(ScalarValue::Utf8(None))])?;
+        match r {
+            ColumnarValue::Scalar(ScalarValue::Utf8(None)) => Ok(()),
+            other => df_execution_err!("Expected null Utf8 scalar, got: {:?}", other),
+        }
+    }
+
     #[test]
     fn test_string_repeat() -> Result<()> {
         // positive case
         let r = string_repeat(&vec![
             ColumnarValue::Array(Arc::new(StringArray::from_iter(vec![
-                Some(format!("123")),
-                Some(format!("a")),
+                Some("123".to_string()),
+                Some("a".to_string()),
                 None,
             ]))),
             ColumnarValue::Scalar(ScalarValue::from(3_i32)),
@@ -390,8 +415,8 @@ mod test {
         // repeat with n < 0
         let r = string_repeat(&vec![
             ColumnarValue::Array(Arc::new(StringArray::from_iter(vec![
-                Some(format!("123")),
-                Some(format!("a")),
+                Some("123".to_string()),
+                Some("a".to_string()),
                 None,
             ]))),
             ColumnarValue::Scalar(ScalarValue::from(-1_i32)),
@@ -405,8 +430,8 @@ mod test {
         // repeat with n = null
         let r = string_repeat(&vec![
             ColumnarValue::Array(Arc::new(StringArray::from_iter(vec![
-                Some(format!("123")),
-                Some(format!("a")),
+                Some("123".to_string()),
+                Some("a".to_string()),
                 None,
             ]))),
             ColumnarValue::Scalar(ScalarValue::Int32(None)),
@@ -424,9 +449,9 @@ mod test {
         // positive case
         let r = string_split(&vec![
             ColumnarValue::Array(Arc::new(StringArray::from_iter(vec![
-                Some(format!("123,456,,,789,")),
-                Some(format!("123")),
-                Some(format!("")),
+                Some("123,456,,,789,".to_string()),
+                Some("123".to_string()),
+                Some(String::new()),
                 None,
             ]))),
             ColumnarValue::Scalar(ScalarValue::from(",")),
@@ -459,16 +484,16 @@ mod test {
         // positive case
         let r = string_concat(&vec![
             ColumnarValue::Array(Arc::new(StringArray::from_iter(vec![
-                Some(format!("123")),
+                Some("123".to_string()),
                 None,
             ]))),
             ColumnarValue::Array(Arc::new(StringArray::from_iter(vec![
-                Some(format!("444")),
-                Some(format!("456")),
+                Some("444".to_string()),
+                Some("456".to_string()),
             ]))),
             ColumnarValue::Array(Arc::new(StringArray::from_iter(vec![
-                Some(format!("")),
-                Some(format!("")),
+                Some(String::new()),
+                Some(String::new()),
             ]))),
             ColumnarValue::Scalar(ScalarValue::from("SomeScalar")),
         ])?;
@@ -485,16 +510,16 @@ mod test {
         let r = string_concat_ws(&vec![
             ColumnarValue::Scalar(ScalarValue::from("||")),
             ColumnarValue::Array(Arc::new(StringArray::from_iter(vec![
-                Some(format!("123")),
+                Some("123".to_string()),
                 None,
             ]))),
             ColumnarValue::Array(Arc::new(StringArray::from_iter(vec![
                 None,
-                Some(format!("456")),
+                Some("456".to_string()),
             ]))),
             ColumnarValue::Array(Arc::new(StringArray::from_iter(vec![
-                Some(format!("")),
-                Some(format!("")),
+                Some(String::new()),
+                Some(String::new()),
             ]))),
             ColumnarValue::Array(Arc::new({
                 let mut list_builder = ListBuilder::new(StringBuilder::new());
