@@ -15,7 +15,7 @@
 
 use std::{
     any::Any,
-    io::{Cursor, Read, Write},
+    io::{Read, Write},
     sync::Arc,
 };
 
@@ -23,8 +23,9 @@ use arrow::{
     array::*,
     datatypes::{DataType, *},
 };
+use auron_memmgr::spill::{SpillCompressedReader, SpillCompressedWriter};
 use bitvec::{bitvec, vec::BitVec};
-use byteorder::{ReadBytesExt, WriteBytesExt};
+use byteorder::WriteBytesExt;
 use datafusion::common::{Result, ScalarValue, utils::proxy::VecAllocExt};
 use datafusion_ext_commons::{
     SliceAsRawBytes, UninitializedInit, df_execution_err, downcast_any,
@@ -33,11 +34,7 @@ use datafusion_ext_commons::{
 };
 use smallvec::SmallVec;
 
-use crate::{
-    agg::agg::IdxSelection,
-    idx_for, idx_with_iter,
-    memmgr::spill::{SpillCompressedReader, SpillCompressedWriter},
-};
+use crate::{agg::agg::IdxSelection, idx_for, idx_with_iter};
 
 pub trait AccColumn: Send {
     fn as_any(&self) -> &dyn Any;
@@ -46,9 +43,9 @@ pub trait AccColumn: Send {
     fn shrink_to_fit(&mut self);
     fn num_records(&self) -> usize;
     fn mem_used(&self) -> usize;
-    fn freeze_to_rows(&self, idx: IdxSelection<'_>, array: &mut [Vec<u8>]) -> Result<()>;
-    fn unfreeze_from_rows(&mut self, cursors: &mut [Cursor<&[u8]>]) -> Result<()>;
-    fn spill(&self, idx: IdxSelection<'_>, w: &mut SpillCompressedWriter) -> Result<()>;
+    fn freeze_to_arrays(&mut self, idx: IdxSelection<'_>) -> Result<Vec<ArrayRef>>;
+    fn unfreeze_from_arrays(&mut self, arrays: &[ArrayRef]) -> Result<()>;
+    fn spill(&mut self, idx: IdxSelection<'_>, w: &mut SpillCompressedWriter) -> Result<()>;
     fn unspill(&mut self, num_rows: usize, r: &mut SpillCompressedReader) -> Result<()>;
 
     fn ensure_size(&mut self, idx: IdxSelection<'_>) {
@@ -140,8 +137,7 @@ impl AccBooleanColumn {
         }
     }
 
-    pub fn to_array(&self, dt: &DataType, idx: IdxSelection<'_>) -> Result<ArrayRef> {
-        assert!(dt == &DataType::Boolean);
+    pub fn to_array(&self, idx: IdxSelection<'_>) -> Result<ArrayRef> {
         idx_with_iter!((idx @ idx) => {
             Ok(Arc::new(BooleanArray::from_iter(
                 idx.map(|i| self.valids[i].then_some(self.values[i]))
@@ -177,38 +173,32 @@ impl AccColumn for AccBooleanColumn {
         self.num_records() / 4 // 2 bits for each value
     }
 
-    fn freeze_to_rows(&self, idx: IdxSelection<'_>, array: &mut [Vec<u8>]) -> Result<()> {
-        idx_with_iter!((idx @ idx) => {
-            for (i, w) in idx.zip(array) {
-                if self.valids[i] {
-                    w.write_u8(1 + self.values[i] as u8)?;
-                } else {
-                    w.write_u8(0)?;
-                }
-            }
-        });
-        Ok(())
+    fn freeze_to_arrays(&mut self, idx: IdxSelection<'_>) -> Result<Vec<ArrayRef>> {
+        let array = self.to_array(idx)?;
+        Ok(vec![array])
     }
 
-    fn unfreeze_from_rows(&mut self, cursors: &mut [Cursor<&[u8]>]) -> Result<()> {
-        self.resize(0);
+    fn unfreeze_from_arrays(&mut self, arrays: &[ArrayRef]) -> Result<()> {
+        let array = downcast_any!(&arrays[0], BooleanArray)?;
+        self.values.clear();
+        self.valids.clear();
 
-        for cursor in cursors {
-            match cursor.read_u8()? {
-                0 => {
-                    self.valids.push(false);
-                    self.values.push(false);
-                }
-                v => {
-                    self.valids.push(true);
-                    self.values.push(v - 1 != 0);
-                }
+        // fill values
+        for i in 0..array.len() {
+            self.values.push(array.value(i));
+        }
+
+        // fill valids
+        if let Some(nb) = array.nulls() {
+            for bit in nb {
+                self.valids.push(bit);
             }
         }
+        self.valids.resize(self.values.len(), true);
         Ok(())
     }
 
-    fn spill(&self, idx: IdxSelection<'_>, w: &mut SpillCompressedWriter) -> Result<()> {
+    fn spill(&mut self, idx: IdxSelection<'_>, w: &mut SpillCompressedWriter) -> Result<()> {
         let mut buf = vec![];
 
         idx_for! {
@@ -243,13 +233,15 @@ impl AccColumn for AccBooleanColumn {
 pub struct AccPrimColumn<T: ArrowNativeType> {
     values: Vec<T>,
     valids: BitVec,
+    dt: DataType,
 }
 
 impl<T: ArrowNativeType> AccPrimColumn<T> {
-    pub fn new(num_records: usize) -> Self {
+    pub fn new(num_records: usize, dt: DataType) -> Self {
         Self {
             values: vec![T::default(); num_records],
             valids: bitvec![0; num_records],
+            dt,
         }
     }
 
@@ -332,39 +324,30 @@ impl<T: ArrowNativeType> AccColumn for AccPrimColumn<T> {
         self.values.allocated_size() + (self.valids.capacity() + 7) / 8
     }
 
-    fn freeze_to_rows(&self, idx: IdxSelection<'_>, array: &mut [Vec<u8>]) -> Result<()> {
-        idx_with_iter!((idx @ idx) => {
-            for (i, w) in idx.zip(array) {
-                if self.valids[i] {
-                    w.write_u8(1)?;
-                    w.write_all([self.values[i]].as_raw_bytes())?;
-                } else {
-                    w.write_u8(0)?;
-                }
-            }
-        });
-        Ok(())
+    fn freeze_to_arrays(&mut self, idx: IdxSelection<'_>) -> Result<Vec<ArrayRef>> {
+        let array = self.to_array(&self.dt, idx)?;
+        Ok(vec![array])
     }
 
-    fn unfreeze_from_rows(&mut self, cursors: &mut [Cursor<&[u8]>]) -> Result<()> {
+    fn unfreeze_from_arrays(&mut self, arrays: &[ArrayRef]) -> Result<()> {
+        let array_data = arrays[0].to_data();
         self.resize(0);
-        let mut value_buf = [T::default()];
 
-        for cursor in cursors {
-            let valid = cursor.read_u8()?;
-            if valid == 1 {
-                cursor.read_exact(value_buf.as_raw_bytes_mut())?;
-                self.values.push(value_buf[0]);
-                self.valids.push(true);
-            } else {
-                self.values.push(T::default());
-                self.valids.push(false);
+        // fill values
+        let buffer = array_data.buffer::<T>(0);
+        self.values = buffer[array_data.offset()..][..array_data.len()].to_vec();
+
+        // fill valids
+        if let Some(nb) = array_data.nulls() {
+            for bit in nb {
+                self.valids.push(bit);
             }
         }
+        self.valids.resize(self.values.len(), true);
         Ok(())
     }
 
-    fn spill(&self, idx: IdxSelection<'_>, w: &mut SpillCompressedWriter) -> Result<()> {
+    fn spill(&mut self, idx: IdxSelection<'_>, w: &mut SpillCompressedWriter) -> Result<()> {
         // write valids
         let mut bits: BitVec<u8> = BitVec::with_capacity(idx.len());
         idx_for! {
@@ -414,13 +397,15 @@ impl<T: ArrowNativeType> AccColumn for AccPrimColumn<T> {
 pub struct AccBytesColumn {
     items: Vec<Option<AccBytes>>,
     heap_mem_used: usize,
+    dt: DataType,
 }
 
 impl AccBytesColumn {
-    pub fn new(num_records: usize) -> Self {
+    pub fn new(num_records: usize, dt: DataType) -> Self {
         Self {
             items: vec![None; num_records],
             heap_mem_used: 0,
+            dt,
         }
     }
 
@@ -439,13 +424,13 @@ impl AccBytesColumn {
         self.heap_mem_used += self.item_heap_mem_used(idx);
     }
 
-    fn to_array(&self, dt: &DataType, idx: IdxSelection<'_>) -> Result<ArrayRef> {
+    fn to_array(&self, idx: IdxSelection<'_>) -> Result<ArrayRef> {
         let binary;
 
         idx_with_iter!((idx @ idx) => {
             binary = BinaryArray::from_iter(idx.map(|i| self.items[i].as_ref()));
         });
-        match dt {
+        match &self.dt {
             DataType::Utf8 => Ok(make_array(
                 binary
                     .to_data()
@@ -454,7 +439,7 @@ impl AccBytesColumn {
                     .build()?,
             )),
             DataType::Binary => Ok(Arc::new(binary)),
-            _ => df_execution_err!("expected string or binary type, got {dt:?}"),
+            dt => df_execution_err!("expected string or binary type, got {dt:?}"),
         }
     }
 
@@ -535,25 +520,34 @@ impl AccColumn for AccBytesColumn {
         self.heap_mem_used + self.items.allocated_size()
     }
 
-    fn freeze_to_rows(&self, idx: IdxSelection<'_>, array: &mut [Vec<u8>]) -> Result<()> {
-        idx_with_iter!((idx @ idx) => {
-            for (i, w) in idx.zip(array) {
-                self.save_value(i, w)?;
-            }
-        });
-        Ok(())
+    fn freeze_to_arrays(&mut self, idx: IdxSelection<'_>) -> Result<Vec<ArrayRef>> {
+        let array = self.to_array(idx)?;
+        Ok(vec![array])
     }
 
-    fn unfreeze_from_rows(&mut self, cursors: &mut [Cursor<&[u8]>]) -> Result<()> {
-        self.items.resize(0, Default::default());
-        for cursor in cursors {
-            self.load_value(cursor)?;
+    fn unfreeze_from_arrays(&mut self, arrays: &[ArrayRef]) -> Result<()> {
+        let array_data = arrays[0].to_data();
+        self.items.clear();
+
+        // fill values
+        let offset_buffer = array_data.buffer::<i32>(0);
+        let data_buffer = array_data.buffer::<u8>(1);
+        for i in 0..array_data.len() {
+            if array_data.is_valid(i) {
+                let offset_begin = offset_buffer[i] as usize;
+                let offset_end = offset_buffer[i + 1] as usize;
+                self.items.push(Some(AccBytes::from_slice(
+                    &data_buffer[offset_begin..offset_end],
+                )));
+            } else {
+                self.items.push(None);
+            }
         }
         self.refresh_heap_mem_used();
         Ok(())
     }
 
-    fn spill(&self, idx: IdxSelection<'_>, w: &mut SpillCompressedWriter) -> Result<()> {
+    fn spill(&mut self, idx: IdxSelection<'_>, w: &mut SpillCompressedWriter) -> Result<()> {
         idx_for! {
             (idx in idx) => {
                 self.save_value(idx, w)?;
@@ -579,17 +573,17 @@ pub struct AccScalarValueColumn {
 }
 
 impl AccScalarValueColumn {
-    pub fn new(dt: &DataType, num_rows: usize) -> Self {
-        let null_value = ScalarValue::try_from(dt).expect("unsupported data type");
+    pub fn new(dt: DataType, num_rows: usize) -> Self {
+        let null_value = ScalarValue::try_from(&dt).expect("unsupported data type");
         Self {
             items: (0..num_rows).map(|_| null_value.clone()).collect(),
-            dt: dt.clone(),
+            dt,
             null_value,
             heap_mem_used: 0,
         }
     }
 
-    pub fn to_array(&mut self, _dt: &DataType, idx: IdxSelection<'_>) -> Result<ArrayRef> {
+    pub fn to_array(&mut self, idx: IdxSelection<'_>) -> Result<ArrayRef> {
         idx_with_iter!((idx @ idx) => {
             ScalarValue::iter_to_array(idx.map(|i| {
                 std::mem::replace(&mut self.items[i], self.null_value.clone())
@@ -645,38 +639,37 @@ impl AccColumn for AccScalarValueColumn {
         self.heap_mem_used + self.items.allocated_size()
     }
 
-    fn freeze_to_rows(&self, idx: IdxSelection<'_>, array: &mut [Vec<u8>]) -> Result<()> {
-        idx_with_iter!((idx @ idx) => {
-            for (i, w) in idx.zip(array) {
-                write_scalar(&self.items[i], true, w)?;
-            }
-        });
-        Ok(())
+    fn freeze_to_arrays(&mut self, idx: IdxSelection<'_>) -> Result<Vec<ArrayRef>> {
+        let array = self.to_array(idx)?; // data type is not used
+        Ok(vec![array])
     }
 
-    fn unfreeze_from_rows(&mut self, cursors: &mut [Cursor<&[u8]>]) -> Result<()> {
-        self.items.truncate(0);
+    fn unfreeze_from_arrays(&mut self, arrays: &[ArrayRef]) -> Result<()> {
+        let array = &arrays[0];
+
+        self.items.clear();
         self.heap_mem_used = 0;
 
-        for cursor in cursors {
-            let scalar = read_scalar(cursor, &self.dt, true)?;
+        for i in 0..array.len() {
+            let scalar = ScalarValue::try_from_array(array, i)?;
             self.heap_mem_used += scalar_value_heap_mem_size(&scalar);
             self.items.push(scalar);
         }
         Ok(())
     }
 
-    fn spill(&self, idx: IdxSelection<'_>, w: &mut SpillCompressedWriter) -> Result<()> {
+    fn spill(&mut self, idx: IdxSelection<'_>, w: &mut SpillCompressedWriter) -> Result<()> {
         idx_for! {
             (idx in idx) => {
-                write_scalar(&self.items[idx], true, w)?;
+                let scalar = self.take_value(idx);
+                write_scalar(&scalar, true, w)?;
             }
         }
         Ok(())
     }
 
     fn unspill(&mut self, num_rows: usize, r: &mut SpillCompressedReader) -> Result<()> {
-        self.items.truncate(0);
+        self.items.clear();
         self.heap_mem_used = 0;
 
         for _ in 0..num_rows {
@@ -688,43 +681,18 @@ impl AccColumn for AccScalarValueColumn {
     }
 }
 
-pub fn create_acc_generic_column(dt: &DataType, num_rows: usize) -> AccColumnRef {
+pub fn create_acc_generic_column(dt: DataType, num_rows: usize) -> AccColumnRef {
     macro_rules! primitive_helper {
         ($t:ty) => {
             Box::new(AccPrimColumn::<<$t as ArrowPrimitiveType>::Native>::new(
-                num_rows,
+                num_rows, dt,
             ))
         };
     }
     downcast_primitive! {
         dt => (primitive_helper),
         DataType::Boolean => Box::new(AccBooleanColumn::new(num_rows)),
-        DataType::Utf8 | DataType::Binary => Box::new(AccBytesColumn::new(num_rows)),
+        DataType::Utf8 | DataType::Binary => Box::new(AccBytesColumn::new(num_rows, dt)),
         other => Box::new(AccScalarValueColumn::new(other, num_rows)),
     }
 }
-
-pub fn acc_generic_column_to_array(
-    column: &mut AccColumnRef,
-    dt: &DataType,
-    idx: IdxSelection<'_>,
-) -> Result<ArrayRef> {
-    macro_rules! primitive_helper {
-        ($t:ty) => {
-            downcast_any!(column, mut AccPrimColumn::<<$t as ArrowPrimitiveType>::Native>)?
-                .to_array(dt, idx)
-        };
-    }
-    downcast_primitive! {
-        dt => (primitive_helper),
-        DataType::Boolean => {
-            downcast_any!(column, mut AccBooleanColumn)?.to_array(dt, idx)
-        }
-        DataType::Utf8 | DataType::Binary => {
-            downcast_any!(column, mut AccBytesColumn)?.to_array(dt, idx)
-        }
-        _other => {
-            downcast_any!(column, mut AccScalarValueColumn)?.to_array(dt, idx)
-        }
-    }
-}
