@@ -24,6 +24,7 @@ use arrow::{
     datatypes::SchemaRef,
 };
 use auron_jni_bridge::conf::{IntConf, UDAF_FALLBACK_NUM_UDAFS_TRIGGER_SORT_AGG};
+use auron_memmgr::MemManager;
 use datafusion::{
     common::{Result, Statistics},
     error::DataFusionError,
@@ -50,7 +51,6 @@ use crate::{
     },
     common::{execution_context::ExecutionContext, timer_helper::TimerHelper},
     expand_exec::ExpandExec,
-    memmgr::MemManager,
     project_exec::ProjectExec,
     sort_exec::create_default_ascending_sort_exec,
 };
@@ -418,6 +418,7 @@ mod test {
         datatypes::{DataType, Field, Schema},
         record_batch::RecordBatch,
     };
+    use auron_memmgr::MemManager;
     use datafusion::{
         assert_batches_sorted_eq,
         common::{Result, ScalarValue},
@@ -435,7 +436,6 @@ mod test {
             agg::create_agg,
         },
         agg_exec::AggExec,
-        memmgr::MemManager,
     };
 
     fn build_table_i32(
@@ -447,7 +447,7 @@ mod test {
         f: (&str, &Vec<i32>),
         g: (&str, &Vec<i32>),
         h: (&str, &Vec<i32>),
-    ) -> RecordBatch {
+    ) -> Result<RecordBatch> {
         let schema = Schema::new(vec![
             Field::new(a.0, DataType::Int32, false),
             Field::new(b.0, DataType::Int32, false),
@@ -459,7 +459,7 @@ mod test {
             Field::new(h.0, DataType::Int32, false),
         ]);
 
-        RecordBatch::try_new(
+        let batch = RecordBatch::try_new(
             Arc::new(schema),
             vec![
                 Arc::new(Int32Array::from(a.1.clone())),
@@ -471,8 +471,8 @@ mod test {
                 Arc::new(Int32Array::from(g.1.clone())),
                 Arc::new(Int32Array::from(h.1.clone())),
             ],
-        )
-        .unwrap()
+        )?;
+        Ok(batch)
     }
 
     fn build_table(
@@ -484,10 +484,11 @@ mod test {
         f: (&str, &Vec<i32>),
         g: (&str, &Vec<i32>),
         h: (&str, &Vec<i32>),
-    ) -> Arc<dyn ExecutionPlan> {
-        let batch = build_table_i32(a, b, c, d, e, f, g, h);
+    ) -> Result<Arc<dyn ExecutionPlan>> {
+        let batch = build_table_i32(a, b, c, d, e, f, g, h)?;
         let schema = batch.schema();
-        Arc::new(TestMemoryExec::try_new(&[vec![batch]], schema, None).unwrap())
+        let exec = TestMemoryExec::try_new(&[vec![batch]], schema, None)?;
+        Ok(Arc::new(exec))
     }
 
     #[tokio::test(flavor = "multi_thread", worker_threads = 1)]
@@ -503,7 +504,7 @@ mod test {
             ("f", &vec![0, 1, 2, 3, 4, 5, 6]),
             ("g", &vec![6, 3, 6, 3, 1, 5, 4]),
             ("h", &vec![6, 3, 6, 3, 1, 5, 4]),
-        );
+        )?;
 
         let agg_expr_sum = create_agg(
             AggFunction::Sum,
@@ -691,6 +692,7 @@ mod fuzztest {
         datatypes::{DataType, Float64Type, Int64Type},
         record_batch::RecordBatch,
     };
+    use auron_memmgr::MemManager;
     use datafusion::{
         common::Result,
         physical_expr::expressions as phys_expr,
@@ -708,7 +710,6 @@ mod fuzztest {
             sum::AggSum,
         },
         agg_exec::AggExec,
-        memmgr::MemManager,
     };
 
     #[tokio::test(flavor = "multi_thread", worker_threads = 1)]
@@ -764,7 +765,7 @@ mod fuzztest {
         let partial_agg = Arc::new(AggExec::try_new(
             HashAgg,
             vec![GroupingExpr {
-                field_name: format!("key"),
+                field_name: "key".to_string(),
                 expr: phys_expr::col("key", &schema)?,
             }],
             vec![
@@ -791,7 +792,7 @@ mod fuzztest {
         let final_agg = Arc::new(AggExec::try_new(
             HashAgg,
             vec![GroupingExpr {
-                field_name: format!("key"),
+                field_name: "key".to_string(),
                 expr: phys_expr::col("key", &schema)?,
             }],
             vec![
