@@ -114,12 +114,10 @@ impl CachedExprsEvaluator {
                                 } else {
                                     Arc::new(uda.filter(selected)?)
                                 }
+                            } else if let Some(previous_selected) = &previous_selected {
+                                filter(&scatter(previous_selected, array)?, selected)?
                             } else {
-                                if let Some(previous_selected) = &previous_selected {
-                                    filter(&scatter(previous_selected, array)?, selected)?
-                                } else {
-                                    filter(&array, selected)?
-                                }
+                                filter(&array, selected)?
                             }
                         })));
                     }
@@ -331,7 +329,7 @@ struct CachedExpr {
 
 impl Display for CachedExpr {
     fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
-        write!(f, "{:?}", self)
+        write!(f, "{self:?}")
     }
 }
 
@@ -402,7 +400,7 @@ impl PhysicalExpr for CachedExpr {
     }
 }
 
-/// A struct holding all evaluated values of cachable expressions
+/// A struct holding all evaluated values of cacheable expressions
 #[derive(Clone)]
 struct Cache {
     values: Arc<Mutex<Vec<Option<ColumnarValue>>>>,
@@ -479,7 +477,7 @@ fn prune_expr_cols(expr: &PhysicalExprRef) -> (PhysicalExprRef, Vec<usize>) {
                 Ok(Transformed::yes(expr))
             }
         })
-        .unwrap()
+        .expect("expr transform failed")
         .data;
 
     let mapped_cols: Vec<usize> = used_cols
