@@ -25,20 +25,19 @@ mod tests {
         datatypes::{DataType, Field, Schema, SchemaRef},
         record_batch::RecordBatch,
     };
+    use auron_memmgr::MemManager;
     use datafusion::{
         assert_batches_sorted_eq,
-        common::JoinSide,
-        error::Result,
+        common::{JoinSide, Result},
         physical_expr::expressions::Column,
         physical_plan::{ExecutionPlan, common, joins::utils::*, test::TestMemoryExec},
-        prelude::SessionContext,
+        prelude::{SessionConfig, SessionContext},
     };
 
     use crate::{
         broadcast_join_build_hash_map_exec::BroadcastJoinBuildHashMapExec,
         broadcast_join_exec::BroadcastJoinExec,
         joins::join_utils::{JoinType, JoinType::*},
-        memmgr::MemManager,
         sort_merge_join_exec::SortMergeJoinExec,
     };
 
@@ -59,44 +58,51 @@ mod tests {
         a: (&str, &Vec<i32>),
         b: (&str, &Vec<i32>),
         c: (&str, &Vec<i32>),
-    ) -> RecordBatch {
+    ) -> Result<RecordBatch> {
         let schema = Schema::new(vec![
             Field::new(a.0, DataType::Int32, false),
             Field::new(b.0, DataType::Int32, false),
             Field::new(c.0, DataType::Int32, false),
         ]);
 
-        RecordBatch::try_new(
+        let batch = RecordBatch::try_new(
             Arc::new(schema),
             vec![
                 Arc::new(Int32Array::from(a.1.clone())),
                 Arc::new(Int32Array::from(b.1.clone())),
                 Arc::new(Int32Array::from(c.1.clone())),
             ],
-        )
-        .unwrap()
+        )?;
+        Ok(batch)
     }
 
     fn build_table(
         a: (&str, &Vec<i32>),
         b: (&str, &Vec<i32>),
         c: (&str, &Vec<i32>),
-    ) -> Arc<dyn ExecutionPlan> {
-        let batch = build_table_i32(a, b, c);
+    ) -> Result<Arc<dyn ExecutionPlan>> {
+        let batch = build_table_i32(a, b, c)?;
         let schema = batch.schema();
-        Arc::new(TestMemoryExec::try_new(&[vec![batch]], schema, None).unwrap())
+        Ok(Arc::new(TestMemoryExec::try_new(
+            &[vec![batch]],
+            schema,
+            None,
+        )?))
     }
 
     fn build_table_from_batches(batches: Vec<RecordBatch>) -> Arc<dyn ExecutionPlan> {
-        let schema = batches.first().unwrap().schema();
-        Arc::new(TestMemoryExec::try_new(&[batches], schema, None).unwrap())
+        let schema = batches.first().expect("missing first batch").schema();
+        Arc::new(
+            TestMemoryExec::try_new(&[batches], schema, None)
+                .expect("failed to create memory exec"),
+        )
     }
 
     fn build_date_table(
         a: (&str, &Vec<i32>),
         b: (&str, &Vec<i32>),
         c: (&str, &Vec<i32>),
-    ) -> Arc<dyn ExecutionPlan> {
+    ) -> Result<Arc<dyn ExecutionPlan>> {
         let schema = Schema::new(vec![
             Field::new(a.0, DataType::Date32, false),
             Field::new(b.0, DataType::Date32, false),
@@ -110,18 +116,21 @@ mod tests {
                 Arc::new(Date32Array::from(b.1.clone())),
                 Arc::new(Date32Array::from(c.1.clone())),
             ],
-        )
-        .unwrap();
+        )?;
 
         let schema = batch.schema();
-        Arc::new(TestMemoryExec::try_new(&[vec![batch]], schema, None).unwrap())
+        Ok(Arc::new(TestMemoryExec::try_new(
+            &[vec![batch]],
+            schema,
+            None,
+        )?))
     }
 
     fn build_date64_table(
         a: (&str, &Vec<i64>),
         b: (&str, &Vec<i64>),
         c: (&str, &Vec<i64>),
-    ) -> Arc<dyn ExecutionPlan> {
+    ) -> Result<Arc<dyn ExecutionPlan>> {
         let schema = Schema::new(vec![
             Field::new(a.0, DataType::Date64, false),
             Field::new(b.0, DataType::Date64, false),
@@ -135,11 +144,14 @@ mod tests {
                 Arc::new(Date64Array::from(b.1.clone())),
                 Arc::new(Date64Array::from(c.1.clone())),
             ],
-        )
-        .unwrap();
+        )?;
 
         let schema = batch.schema();
-        Arc::new(TestMemoryExec::try_new(&[vec![batch]], schema, None).unwrap())
+        Ok(Arc::new(TestMemoryExec::try_new(
+            &[vec![batch]],
+            schema,
+            None,
+        )?))
     }
 
     /// returns a table with 3 columns of i32 in memory
@@ -147,7 +159,7 @@ mod tests {
         a: (&str, &Vec<Option<i32>>),
         b: (&str, &Vec<Option<i32>>),
         c: (&str, &Vec<Option<i32>>),
-    ) -> Arc<dyn ExecutionPlan> {
+    ) -> Result<Arc<dyn ExecutionPlan>> {
         let schema = Arc::new(Schema::new(vec![
             Field::new(a.0, DataType::Int32, true),
             Field::new(b.0, DataType::Int32, true),
@@ -160,9 +172,12 @@ mod tests {
                 Arc::new(Int32Array::from(b.1.clone())),
                 Arc::new(Int32Array::from(c.1.clone())),
             ],
-        )
-        .unwrap();
-        Arc::new(TestMemoryExec::try_new(&[vec![batch]], schema, None).unwrap())
+        )?;
+        Ok(Arc::new(TestMemoryExec::try_new(
+            &[vec![batch]],
+            schema,
+            None,
+        )?))
     }
 
     fn build_join_schema_for_test(
@@ -219,6 +234,7 @@ mod tests {
                     JoinSide::Right,
                     true,
                     None,
+                    false,
                 )?)
             }
             BHJRightProbed => {
@@ -235,6 +251,7 @@ mod tests {
                     JoinSide::Left,
                     true,
                     None,
+                    false,
                 )?)
             }
             SHJLeftProbed => Arc::new(BroadcastJoinExec::try_new(
@@ -246,6 +263,7 @@ mod tests {
                 JoinSide::Right,
                 false,
                 None,
+                false,
             )?),
             SHJRightProbed => Arc::new(BroadcastJoinExec::try_new(
                 schema,
@@ -256,6 +274,96 @@ mod tests {
                 JoinSide::Left,
                 false,
                 None,
+                false,
+            )?),
+        };
+        let columns = columns(&join.schema());
+        let stream = join.execute(0, task_ctx)?;
+        let batches = common::collect(stream).await?;
+        Ok((columns, batches))
+    }
+
+    async fn join_collect_with_batch_size(
+        test_type: TestType,
+        left: Arc<dyn ExecutionPlan>,
+        right: Arc<dyn ExecutionPlan>,
+        on: JoinOn,
+        join_type: JoinType,
+        batch_size: usize,
+    ) -> Result<(Vec<String>, Vec<RecordBatch>)> {
+        MemManager::init(1000000);
+        let session_config = SessionConfig::new().with_batch_size(batch_size);
+        let session_ctx = SessionContext::new_with_config(session_config);
+        let task_ctx = session_ctx.task_ctx();
+        let schema = build_join_schema_for_test(&left.schema(), &right.schema(), join_type)?;
+
+        let join: Arc<dyn ExecutionPlan> = match test_type {
+            SMJ => {
+                let sort_options = vec![SortOptions::default(); on.len()];
+                Arc::new(SortMergeJoinExec::try_new(
+                    schema,
+                    left,
+                    right,
+                    on,
+                    join_type,
+                    sort_options,
+                )?)
+            }
+            BHJLeftProbed => {
+                let right = Arc::new(BroadcastJoinBuildHashMapExec::new(
+                    right,
+                    on.iter().map(|(_, right_key)| right_key.clone()).collect(),
+                ));
+                Arc::new(BroadcastJoinExec::try_new(
+                    schema,
+                    left,
+                    right,
+                    on,
+                    join_type,
+                    JoinSide::Right,
+                    true,
+                    None,
+                    false,
+                )?)
+            }
+            BHJRightProbed => {
+                let left = Arc::new(BroadcastJoinBuildHashMapExec::new(
+                    left,
+                    on.iter().map(|(left_key, _)| left_key.clone()).collect(),
+                ));
+                Arc::new(BroadcastJoinExec::try_new(
+                    schema,
+                    left,
+                    right,
+                    on,
+                    join_type,
+                    JoinSide::Left,
+                    true,
+                    None,
+                    false,
+                )?)
+            }
+            SHJLeftProbed => Arc::new(BroadcastJoinExec::try_new(
+                schema,
+                left,
+                right,
+                on,
+                join_type,
+                JoinSide::Right,
+                false,
+                None,
+                false,
+            )?),
+            SHJRightProbed => Arc::new(BroadcastJoinExec::try_new(
+                schema,
+                left,
+                right,
+                on,
+                join_type,
+                JoinSide::Left,
+                false,
+                None,
+                false,
             )?),
         };
         let columns = columns(&join.schema());
@@ -279,12 +387,12 @@ mod tests {
                 ("a1", &vec![1, 2, 3]),
                 ("b1", &vec![4, 5, 5]), // this has a repetition
                 ("c1", &vec![7, 8, 9]),
-            );
+            )?;
             let right = build_table(
                 ("a2", &vec![10, 20, 30]),
                 ("b1", &vec![4, 5, 6]),
                 ("c2", &vec![70, 80, 90]),
-            );
+            )?;
 
             let on: JoinOn = vec![(
                 Arc::new(Column::new_with_schema("b1", &left.schema())?),
@@ -314,12 +422,12 @@ mod tests {
                 ("a1", &vec![1, 2, 2]),
                 ("b2", &vec![1, 2, 2]),
                 ("c1", &vec![7, 8, 9]),
-            );
+            )?;
             let right = build_table(
                 ("a1", &vec![1, 2, 3]),
                 ("b2", &vec![1, 2, 2]),
                 ("c2", &vec![70, 80, 90]),
-            );
+            )?;
             let on: JoinOn = vec![
                 (
                     Arc::new(Column::new_with_schema("a1", &left.schema())?),
@@ -354,12 +462,12 @@ mod tests {
                 ("a1", &vec![1, 1, 2]),
                 ("b2", &vec![1, 1, 2]),
                 ("c1", &vec![7, 8, 9]),
-            );
+            )?;
             let right = build_table(
                 ("a1", &vec![1, 1, 3]),
                 ("b2", &vec![1, 1, 2]),
                 ("c2", &vec![70, 80, 90]),
-            );
+            )?;
             let on: JoinOn = vec![
                 (
                     Arc::new(Column::new_with_schema("a1", &left.schema())?),
@@ -395,12 +503,12 @@ mod tests {
                 ("a1", &vec![Some(1), Some(1), Some(2), Some(2)]),
                 ("b2", &vec![None, Some(1), Some(2), Some(2)]), // null in key field
                 ("c1", &vec![Some(1), None, Some(8), Some(9)]), // null in non-key field
-            );
+            )?;
             let right = build_table_i32_nullable(
                 ("a1", &vec![Some(1), Some(1), Some(2), Some(3)]),
                 ("b2", &vec![None, Some(1), Some(2), Some(2)]),
                 ("c2", &vec![Some(10), Some(70), Some(80), Some(90)]),
-            );
+            )?;
             let on: JoinOn = vec![
                 (
                     Arc::new(Column::new_with_schema("a1", &left.schema())?),
@@ -428,6 +536,118 @@ mod tests {
         Ok(())
     }
 
+    #[tokio::test(flavor = "multi_thread", worker_threads = 1)]
+    async fn join_inner_batchsize() -> Result<()> {
+        for test_type in ALL_TEST_TYPE {
+            let left = build_table(
+                ("a1", &vec![1, 1, 1, 1, 1]),
+                ("b1", &vec![1, 2, 3, 4, 5]),
+                ("c1", &vec![1, 2, 3, 4, 5]),
+            )?;
+            let right = build_table(
+                ("a2", &vec![1, 1, 1, 1, 1, 1, 1]),
+                ("b2", &vec![1, 2, 3, 4, 5, 6, 7]),
+                ("c2", &vec![1, 2, 3, 4, 5, 6, 7]),
+            )?;
+            let on: JoinOn = vec![(
+                Arc::new(Column::new_with_schema("a1", &left.schema())?),
+                Arc::new(Column::new_with_schema("a2", &right.schema())?),
+            )];
+            let expected = vec![
+                "+----+----+----+----+----+----+",
+                "| a1 | b1 | c1 | a2 | b2 | c2 |",
+                "+----+----+----+----+----+----+",
+                "| 1  | 1  | 1  | 1  | 1  | 1  |",
+                "| 1  | 1  | 1  | 1  | 2  | 2  |",
+                "| 1  | 1  | 1  | 1  | 3  | 3  |",
+                "| 1  | 1  | 1  | 1  | 4  | 4  |",
+                "| 1  | 1  | 1  | 1  | 5  | 5  |",
+                "| 1  | 1  | 1  | 1  | 6  | 6  |",
+                "| 1  | 1  | 1  | 1  | 7  | 7  |",
+                "| 1  | 2  | 2  | 1  | 1  | 1  |",
+                "| 1  | 2  | 2  | 1  | 2  | 2  |",
+                "| 1  | 2  | 2  | 1  | 3  | 3  |",
+                "| 1  | 2  | 2  | 1  | 4  | 4  |",
+                "| 1  | 2  | 2  | 1  | 5  | 5  |",
+                "| 1  | 2  | 2  | 1  | 6  | 6  |",
+                "| 1  | 2  | 2  | 1  | 7  | 7  |",
+                "| 1  | 3  | 3  | 1  | 1  | 1  |",
+                "| 1  | 3  | 3  | 1  | 2  | 2  |",
+                "| 1  | 3  | 3  | 1  | 3  | 3  |",
+                "| 1  | 3  | 3  | 1  | 4  | 4  |",
+                "| 1  | 3  | 3  | 1  | 5  | 5  |",
+                "| 1  | 3  | 3  | 1  | 6  | 6  |",
+                "| 1  | 3  | 3  | 1  | 7  | 7  |",
+                "| 1  | 4  | 4  | 1  | 1  | 1  |",
+                "| 1  | 4  | 4  | 1  | 2  | 2  |",
+                "| 1  | 4  | 4  | 1  | 3  | 3  |",
+                "| 1  | 4  | 4  | 1  | 4  | 4  |",
+                "| 1  | 4  | 4  | 1  | 5  | 5  |",
+                "| 1  | 4  | 4  | 1  | 6  | 6  |",
+                "| 1  | 4  | 4  | 1  | 7  | 7  |",
+                "| 1  | 5  | 5  | 1  | 1  | 1  |",
+                "| 1  | 5  | 5  | 1  | 2  | 2  |",
+                "| 1  | 5  | 5  | 1  | 3  | 3  |",
+                "| 1  | 5  | 5  | 1  | 4  | 4  |",
+                "| 1  | 5  | 5  | 1  | 5  | 5  |",
+                "| 1  | 5  | 5  | 1  | 6  | 6  |",
+                "| 1  | 5  | 5  | 1  | 7  | 7  |",
+                "+----+----+----+----+----+----+",
+            ];
+            let (_, batches) = join_collect_with_batch_size(
+                test_type,
+                left.clone(),
+                right.clone(),
+                on.clone(),
+                Inner,
+                2,
+            )
+            .await?;
+            assert_batches_sorted_eq!(expected, &batches);
+            let (_, batches) = join_collect_with_batch_size(
+                test_type,
+                left.clone(),
+                right.clone(),
+                on.clone(),
+                Inner,
+                3,
+            )
+            .await?;
+            assert_batches_sorted_eq!(expected, &batches);
+            let (_, batches) = join_collect_with_batch_size(
+                test_type,
+                left.clone(),
+                right.clone(),
+                on.clone(),
+                Inner,
+                4,
+            )
+            .await?;
+            assert_batches_sorted_eq!(expected, &batches);
+            let (_, batches) = join_collect_with_batch_size(
+                test_type,
+                left.clone(),
+                right.clone(),
+                on.clone(),
+                Inner,
+                5,
+            )
+            .await?;
+            assert_batches_sorted_eq!(expected, &batches);
+            let (_, batches) = join_collect_with_batch_size(
+                test_type,
+                left.clone(),
+                right.clone(),
+                on.clone(),
+                Inner,
+                7,
+            )
+            .await?;
+            assert_batches_sorted_eq!(expected, &batches);
+        }
+        Ok(())
+    }
+
     #[tokio::test(flavor = "multi_thread", worker_threads = 1)]
     async fn join_left_one() -> Result<()> {
         for test_type in ALL_TEST_TYPE {
@@ -435,12 +655,12 @@ mod tests {
                 ("a1", &vec![1, 2, 3]),
                 ("b1", &vec![4, 5, 7]), // 7 does not exist on the right
                 ("c1", &vec![7, 8, 9]),
-            );
+            )?;
             let right = build_table(
                 ("a2", &vec![10, 20, 30]),
                 ("b1", &vec![4, 5, 6]),
                 ("c2", &vec![70, 80, 90]),
-            );
+            )?;
             let on: JoinOn = vec![(
                 Arc::new(Column::new_with_schema("b1", &left.schema())?),
                 Arc::new(Column::new_with_schema("b1", &right.schema())?),
@@ -469,12 +689,12 @@ mod tests {
                 ("a1", &vec![1, 2, 3]),
                 ("b1", &vec![4, 5, 7]),
                 ("c1", &vec![7, 8, 9]),
-            );
+            )?;
             let right = build_table(
                 ("a2", &vec![10, 20, 30]),
                 ("b1", &vec![4, 5, 6]), // 6 does not exist on the left
                 ("c2", &vec![70, 80, 90]),
-            );
+            )?;
             let on: JoinOn = vec![(
                 Arc::new(Column::new_with_schema("b1", &left.schema())?),
                 Arc::new(Column::new_with_schema("b1", &right.schema())?),
@@ -503,12 +723,12 @@ mod tests {
                 ("a1", &vec![1, 2, 2, 3]),
                 ("b1", &vec![4, 5, 5, 7]), // 7 does not exist on the right
                 ("c1", &vec![7, 8, 80, 9]),
-            );
+            )?;
             let right = build_table(
                 ("a2", &vec![10, 20, 20, 30]),
                 ("b2", &vec![4, 5, 5, 6]),
                 ("c2", &vec![70, 80, 800, 90]),
-            );
+            )?;
             let on: JoinOn = vec![(
                 Arc::new(Column::new_with_schema("b1", &left.schema())?),
                 Arc::new(Column::new_with_schema("b2", &right.schema())?),
@@ -540,12 +760,12 @@ mod tests {
                 ("a1", &vec![1, 2, 2, 3, 5]),
                 ("b1", &vec![4, 5, 5, 7, 7]), // 7 does not exist on the right
                 ("c1", &vec![7, 8, 8, 9, 11]),
-            );
+            )?;
             let right = build_table(
                 ("a2", &vec![10, 20, 30]),
                 ("b1", &vec![4, 5, 6]),
                 ("c2", &vec![70, 80, 90]),
-            );
+            )?;
             let on: JoinOn = vec![(
                 Arc::new(Column::new_with_schema("b1", &left.schema())?),
                 Arc::new(Column::new_with_schema("b1", &right.schema())?),
@@ -573,12 +793,12 @@ mod tests {
                 ("a1", &vec![1, 2, 2, 3]),
                 ("b1", &vec![4, 5, 5, 7]), // 7 does not exist on the right
                 ("c1", &vec![7, 8, 8, 9]),
-            );
+            )?;
             let right = build_table(
                 ("a2", &vec![10, 20, 30]),
                 ("b1", &vec![4, 5, 6]), // 5 is double on the right
                 ("c2", &vec![70, 80, 90]),
-            );
+            )?;
             let on: JoinOn = vec![(
                 Arc::new(Column::new_with_schema("b1", &left.schema())?),
                 Arc::new(Column::new_with_schema("b1", &right.schema())?),
@@ -600,6 +820,40 @@ mod tests {
         Ok(())
     }
 
+    #[tokio::test(flavor = "multi_thread", worker_threads = 1)]
+    async fn join_anti_with_null_keys() -> Result<()> {
+        let left = build_table_i32_nullable(
+            ("a1", &vec![Some(1), Some(2), None, Some(4), Some(5)]),
+            ("b1", &vec![Some(4), Some(5), Some(6), None, Some(8)]),
+            ("c1", &vec![Some(7), Some(8), Some(9), Some(10), Some(11)]),
+        )?;
+        let right = build_table_i32_nullable(
+            ("a2", &vec![Some(10), Some(20), Some(30)]),
+            ("b1", &vec![Some(4), Some(5), Some(7)]),
+            ("c2", &vec![Some(70), Some(80), Some(90)]),
+        )?;
+        let on: JoinOn = vec![(
+            Arc::new(Column::new_with_schema("b1", &left.schema())?),
+            Arc::new(Column::new_with_schema("b1", &right.schema())?),
+        )];
+
+        for test_type in ALL_TEST_TYPE {
+            let (_, batches) =
+                join_collect(test_type, left.clone(), right.clone(), on.clone(), LeftAnti).await?;
+            let expected = vec![
+                "+----+----+----+",
+                "| a1 | b1 | c1 |",
+                "+----+----+----+",
+                "|    | 6  | 9  |",
+                "| 4  |    | 10 |",
+                "| 5  | 8  | 11 |",
+                "+----+----+----+",
+            ];
+            assert_batches_sorted_eq!(expected, &batches);
+        }
+        Ok(())
+    }
+
     #[tokio::test(flavor = "multi_thread", worker_threads = 1)]
     async fn join_with_duplicated_column_names() -> Result<()> {
         for test_type in ALL_TEST_TYPE {
@@ -607,12 +861,12 @@ mod tests {
                 ("a", &vec![1, 2, 3]),
                 ("b", &vec![4, 5, 7]),
                 ("c", &vec![7, 8, 9]),
-            );
+            )?;
             let right = build_table(
                 ("a", &vec![10, 20, 30]),
                 ("b", &vec![1, 2, 7]),
                 ("c", &vec![70, 80, 90]),
-            );
+            )?;
             let on: JoinOn = vec![(
                 // join on a=b so there are duplicate column names on unjoined columns
                 Arc::new(Column::new_with_schema("a", &left.schema())?),
@@ -641,12 +895,12 @@ mod tests {
                 ("a1", &vec![1, 2, 3]),
                 ("b1", &vec![19107, 19108, 19108]), // this has a repetition
                 ("c1", &vec![7, 8, 9]),
-            );
+            )?;
             let right = build_date_table(
                 ("a2", &vec![10, 20, 30]),
                 ("b1", &vec![19107, 19108, 19109]),
                 ("c2", &vec![70, 80, 90]),
-            );
+            )?;
 
             let on: JoinOn = vec![(
                 Arc::new(Column::new_with_schema("b1", &left.schema())?),
@@ -678,12 +932,12 @@ mod tests {
                 ("b1", &vec![1650703441000, 1650903441000, 1650903441000]), /* this has a
                                                                              * repetition */
                 ("c1", &vec![7, 8, 9]),
-            );
+            )?;
             let right = build_date64_table(
                 ("a2", &vec![10, 20, 30]),
                 ("b1", &vec![1650703441000, 1650503441000, 1650903441000]),
                 ("c2", &vec![70, 80, 90]),
-            );
+            )?;
 
             let on: JoinOn = vec![(
                 Arc::new(Column::new_with_schema("b1", &left.schema())?),
@@ -714,12 +968,12 @@ mod tests {
                 ("a1", &vec![0, 1, 2, 3, 4, 5]),
                 ("b1", &vec![3, 4, 5, 6, 6, 7]),
                 ("c1", &vec![4, 5, 6, 7, 8, 9]),
-            );
+            )?;
             let right = build_table(
                 ("a2", &vec![0, 10, 20, 30, 40]),
                 ("b2", &vec![2, 4, 6, 6, 8]),
                 ("c2", &vec![50, 60, 70, 80, 90]),
-            );
+            )?;
             let on: JoinOn = vec![(
                 Arc::new(Column::new_with_schema("b1", &left.schema())?),
                 Arc::new(Column::new_with_schema("b2", &right.schema())?),
@@ -752,12 +1006,12 @@ mod tests {
                 ("a1", &vec![0, 1, 2, 3]),
                 ("b1", &vec![3, 4, 5, 7]),
                 ("c1", &vec![6, 7, 8, 9]),
-            );
+            )?;
             let right = build_table(
                 ("a2", &vec![0, 10, 20, 30]),
                 ("b2", &vec![2, 4, 5, 6]),
                 ("c2", &vec![60, 70, 80, 90]),
-            );
+            )?;
             let on: JoinOn = vec![(
                 Arc::new(Column::new_with_schema("b1", &left.schema())?),
                 Arc::new(Column::new_with_schema("b2", &right.schema())?),
@@ -786,22 +1040,22 @@ mod tests {
                 ("a1", &vec![0, 1, 2]),
                 ("b1", &vec![3, 4, 5]),
                 ("c1", &vec![4, 5, 6]),
-            );
+            )?;
             let left_batch_2 = build_table_i32(
                 ("a1", &vec![3, 4, 5, 6]),
                 ("b1", &vec![6, 6, 7, 9]),
                 ("c1", &vec![7, 8, 9, 9]),
-            );
+            )?;
             let right_batch_1 = build_table_i32(
                 ("a2", &vec![0, 10, 20]),
                 ("b2", &vec![2, 4, 6]),
                 ("c2", &vec![50, 60, 70]),
-            );
+            )?;
             let right_batch_2 = build_table_i32(
                 ("a2", &vec![30, 40]),
                 ("b2", &vec![6, 8]),
                 ("c2", &vec![80, 90]),
-            );
+            )?;
             let left = build_table_from_batches(vec![left_batch_1, left_batch_2]);
             let right = build_table_from_batches(vec![right_batch_1, right_batch_2]);
             let on: JoinOn = vec![(
@@ -837,22 +1091,22 @@ mod tests {
                 ("a2", &vec![0, 1, 2]),
                 ("b2", &vec![3, 4, 5]),
                 ("c2", &vec![4, 5, 6]),
-            );
+            )?;
             let right_batch_2 = build_table_i32(
                 ("a2", &vec![3, 4, 5, 6]),
                 ("b2", &vec![6, 6, 7, 9]),
                 ("c2", &vec![7, 8, 9, 9]),
-            );
+            )?;
             let left_batch_1 = build_table_i32(
                 ("a1", &vec![0, 10, 20]),
                 ("b1", &vec![2, 4, 6]),
                 ("c1", &vec![50, 60, 70]),
-            );
+            )?;
             let left_batch_2 = build_table_i32(
                 ("a1", &vec![30, 40]),
                 ("b1", &vec![6, 8]),
                 ("c1", &vec![80, 90]),
-            );
+            )?;
             let left = build_table_from_batches(vec![left_batch_1, left_batch_2]);
             let right = build_table_from_batches(vec![right_batch_1, right_batch_2]);
             let on: JoinOn = vec![(
@@ -888,22 +1142,22 @@ mod tests {
                 ("a1", &vec![0, 1, 2]),
                 ("b1", &vec![3, 4, 5]),
                 ("c1", &vec![4, 5, 6]),
-            );
+            )?;
             let left_batch_2 = build_table_i32(
                 ("a1", &vec![3, 4, 5, 6]),
                 ("b1", &vec![6, 6, 7, 9]),
                 ("c1", &vec![7, 8, 9, 9]),
-            );
+            )?;
             let right_batch_1 = build_table_i32(
                 ("a2", &vec![0, 10, 20]),
                 ("b2", &vec![2, 4, 6]),
                 ("c2", &vec![50, 60, 70]),
-            );
+            )?;
             let right_batch_2 = build_table_i32(
                 ("a2", &vec![30, 40]),
                 ("b2", &vec![6, 8]),
                 ("c2", &vec![80, 90]),
-            );
+            )?;
             let left = build_table_from_batches(vec![left_batch_1, left_batch_2]);
             let right = build_table_from_batches(vec![right_batch_1, right_batch_2]);
             let on: JoinOn = vec![(
@@ -941,22 +1195,22 @@ mod tests {
                 ("a1", &vec![0, 1, 2]),
                 ("b1", &vec![3, 4, 5]),
                 ("c1", &vec![4, 5, 6]),
-            );
+            )?;
             let left_batch_2 = build_table_i32(
                 ("a1", &vec![3, 4, 5, 6]),
                 ("b1", &vec![6, 6, 7, 9]),
                 ("c1", &vec![7, 8, 9, 9]),
-            );
+            )?;
             let right_batch_1 = build_table_i32(
                 ("a2", &vec![0, 10, 20]),
                 ("b2", &vec![2, 4, 6]),
                 ("c2", &vec![50, 60, 70]),
-            );
+            )?;
             let right_batch_2 = build_table_i32(
                 ("a2", &vec![30, 40]),
                 ("b2", &vec![6, 8]),
                 ("c2", &vec![80, 90]),
-            );
+            )?;
             let left = build_table_from_batches(vec![left_batch_1, left_batch_2]);
             let right = build_table_from_batches(vec![right_batch_1, right_batch_2]);
             let on: JoinOn = vec![(
