@@ -27,6 +27,7 @@ use datafusion_ext_commons::{
         selection::{BatchInterleaver, create_batch_interleaver},
     },
     compute_suggested_batch_size_for_output, df_execution_err,
+    io::ipc_compression::IpcCompressionWriter,
 };
 use itertools::Itertools;
 use jni::objects::GlobalRef;
@@ -35,7 +36,6 @@ use parking_lot::Mutex;
 
 use crate::{
     common::{
-        ipc_compression::IpcCompressionWriter,
         offsetted::{Offsetted, OffsettedMergeIterator},
         timer_helper::TimerHelper,
     },
@@ -246,6 +246,7 @@ impl<'a> PartitionedBatchesIterator<'a> {
     }
 
     /// all iterators returned should have been fully consumed
+    #[allow(clippy::panic)] // Temporarily allow panic to refactor to Result later
     pub fn next_partition_chunk(
         &mut self,
     ) -> Option<(usize, impl Iterator<Item = RecordBatch> + 'a)> {
@@ -311,7 +312,8 @@ fn sort_batches_by_partition_id(
                     part_ids
                 }
                 Partitioning::RangePartitioning(sort_expr, _, bounds) => {
-                    evaluate_range_partition_ids(&batch, sort_expr, bounds).unwrap()
+                    evaluate_range_partition_ids(&batch, sort_expr, bounds)
+                        .expect("failed to evaluate range partition ids")
                 }
                 _ => unreachable!("unsupported partitioning: {:?}", partitioning),
             };
@@ -373,22 +375,22 @@ mod test {
         a: (&str, &Vec<i32>),
         b: (&str, &Vec<i32>),
         c: (&str, &Vec<i32>),
-    ) -> RecordBatch {
+    ) -> Result<RecordBatch> {
         let schema = Schema::new(vec![
             Field::new(a.0, DataType::Int32, false),
             Field::new(b.0, DataType::Int32, false),
             Field::new(c.0, DataType::Int32, false),
         ]);
 
-        RecordBatch::try_new(
+        let batch = RecordBatch::try_new(
             Arc::new(schema),
             vec![
                 Arc::new(Int32Array::from(a.1.clone())),
                 Arc::new(Int32Array::from(b.1.clone())),
                 Arc::new(Int32Array::from(c.1.clone())),
             ],
-        )
-        .unwrap()
+        )?;
+        Ok(batch)
     }
 
     #[tokio::test]
@@ -397,7 +399,7 @@ mod test {
             ("a", &vec![19, 18, 17, 16, 15, 14, 13, 12, 11, 10]),
             ("b", &vec![0, 1, 2, 3, 4, 5, 6, 7, 8, 9]),
             ("c", &vec![5, 6, 7, 8, 9, 0, 1, 2, 3, 4]),
-        );
+        )?;
 
         let round_robin_partitioning = Partitioning::RoundRobinPartitioning(4);
         let (_parts, sorted_batch) =
@@ -429,7 +431,7 @@ mod test {
             ("a", &vec![19, 18, 17, 16, 15, 14, 13, 12, 11, 10]),
             ("b", &vec![0, 1, 2, 3, 4, 5, 6, 7, 8, 9]),
             ("c", &vec![5, 6, 7, 8, 9, 0, 1, 2, 3, 4]),
-        );
+        )?;
         let sort_exprs = vec![PhysicalSortExpr {
             expr: Arc::new(Column::new("a", 0)),
             options: SortOptions::default(),
@@ -449,7 +451,7 @@ mod test {
                 .collect::<Result<Vec<SortField>>>()?,
         )?));
 
-        let rows: Rows = sort_row_converter.lock().convert_columns(&bounds).unwrap();
+        let rows: Rows = sort_row_converter.lock().convert_columns(&bounds)?;
         let partition_num = rows.num_rows() + 1;
 
         let range_repartitioning =
@@ -483,7 +485,7 @@ mod test {
             ("a", &vec![19, 18, 17, 16, 15, 14, 13, 12, 11, 10]),
             ("b", &vec![0, 1, 2, 3, 4, 5, 6, 7, 8, 9]),
             ("c", &vec![5, 6, 7, 8, 9, 0, 1, 2, 3, 4]),
-        );
+        )?;
         let sort_exprs = vec![
             PhysicalSortExpr {
                 expr: Arc::new(Column::new("a", 0)),
@@ -511,7 +513,7 @@ mod test {
                 .collect::<Result<Vec<SortField>>>()?,
         )?));
 
-        let rows: Rows = sort_row_converter.lock().convert_columns(&bounds).unwrap();
+        let rows: Rows = sort_row_converter.lock().convert_columns(&bounds)?;
         let partition_num = rows.num_rows() + 1;
 
         let range_repartitioning =
