@@ -26,14 +26,10 @@ import org.apache.commons.lang3.reflect.MethodUtils
 import org.apache.hadoop.hive.ql.io.parquet.MapredParquetOutputFormat
 import org.apache.spark.Partition
 import org.apache.spark.broadcast.Broadcast
-import org.apache.spark.internal.Logging
-import org.apache.spark.sql.auron.AuronConvertStrategy.childOrderingRequiredTag
-import org.apache.spark.sql.auron.AuronConvertStrategy.convertibleTag
-import org.apache.spark.sql.auron.AuronConvertStrategy.convertStrategyTag
-import org.apache.spark.sql.auron.AuronConvertStrategy.convertToNonNativeTag
-import org.apache.spark.sql.auron.AuronConvertStrategy.isNeverConvert
-import org.apache.spark.sql.auron.AuronConvertStrategy.joinSmallerSideTag
-import org.apache.spark.sql.auron.NativeConverters.{roundRobinTypeSupported, scalarTypeSupported, StubExpr}
+import org.apache.spark.internal.{config, Logging}
+import org.apache.spark.sql.auron.AuronConvertStrategy.{childOrderingRequiredTag, convertibleTag, convertStrategyTag, convertToNonNativeTag, isNeverConvert, joinSmallerSideTag, neverConvertReasonTag}
+import org.apache.spark.sql.auron.NativeConverters.{existTimestampType, isTypeSupported, roundRobinTypeSupported, StubExpr}
+import org.apache.spark.sql.auron.join.JoinBuildSides.{JoinBuildLeft, JoinBuildRight, JoinBuildSide}
 import org.apache.spark.sql.auron.util.AuronLogUtils.logDebugPlanConversion
 import org.apache.spark.sql.catalyst.expressions.AggregateWindowFunction
 import org.apache.spark.sql.catalyst.expressions.Alias
@@ -54,25 +50,10 @@ import org.apache.spark.sql.catalyst.plans.physical.HashPartitioning
 import org.apache.spark.sql.catalyst.plans.physical.Partitioning
 import org.apache.spark.sql.catalyst.plans.physical.RangePartitioning
 import org.apache.spark.sql.catalyst.plans.physical.RoundRobinPartitioning
-import org.apache.spark.sql.execution.ExpandExec
-import org.apache.spark.sql.execution.FileSourceScanExec
-import org.apache.spark.sql.execution.FilterExec
-import org.apache.spark.sql.execution.GenerateExec
-import org.apache.spark.sql.execution.GlobalLimitExec
-import org.apache.spark.sql.execution.LeafExecNode
-import org.apache.spark.sql.execution.LocalLimitExec
-import org.apache.spark.sql.execution.LocalTableScanExec
-import org.apache.spark.sql.execution.ProjectExec
-import org.apache.spark.sql.execution.SortExec
-import org.apache.spark.sql.execution.SparkPlan
-import org.apache.spark.sql.execution.TakeOrderedAndProjectExec
-import org.apache.spark.sql.execution.UnaryExecNode
-import org.apache.spark.sql.execution.UnionExec
+import org.apache.spark.sql.execution._
 import org.apache.spark.sql.execution.aggregate.HashAggregateExec
 import org.apache.spark.sql.execution.aggregate.ObjectHashAggregateExec
 import org.apache.spark.sql.execution.aggregate.SortAggregateExec
-import org.apache.spark.sql.execution.auron.plan.BroadcastLeft
-import org.apache.spark.sql.execution.auron.plan.BroadcastRight
 import org.apache.spark.sql.execution.auron.plan.ConvertToNativeBase
 import org.apache.spark.sql.execution.auron.plan.NativeAggBase
 import org.apache.spark.sql.execution.auron.plan.NativeBroadcastExchangeBase
@@ -91,66 +72,63 @@ import org.apache.spark.sql.hive.execution.auron.plan.NativeHiveTableScanBase
 import org.apache.spark.sql.internal.SQLConf
 import org.apache.spark.sql.types.LongType
 
+import org.apache.auron.configuration.AuronConfiguration
+import org.apache.auron.jni.AuronAdaptor
+import org.apache.auron.metric.SparkMetricNode
 import org.apache.auron.protobuf.EmptyPartitionsExecNode
 import org.apache.auron.protobuf.PhysicalPlanNode
-import org.apache.auron.sparkver
+import org.apache.auron.spark.configuration.SparkAuronConfiguration
 
 object AuronConverters extends Logging {
-  def enableScan: Boolean =
-    getBooleanConf("spark.auron.enable.scan", defaultValue = true)
-  def enableProject: Boolean =
-    getBooleanConf("spark.auron.enable.project", defaultValue = true)
-  def enableFilter: Boolean =
-    getBooleanConf("spark.auron.enable.filter", defaultValue = true)
-  def enableSort: Boolean =
-    getBooleanConf("spark.auron.enable.sort", defaultValue = true)
-  def enableUnion: Boolean =
-    getBooleanConf("spark.auron.enable.union", defaultValue = true)
-  def enableSmj: Boolean =
-    getBooleanConf("spark.auron.enable.smj", defaultValue = true)
-  def enableShj: Boolean =
-    getBooleanConf("spark.auron.enable.shj", defaultValue = true)
-  def enableBhj: Boolean =
-    getBooleanConf("spark.auron.enable.bhj", defaultValue = true)
-  def enableBnlj: Boolean =
-    getBooleanConf("spark.auron.enable.bnlj", defaultValue = true)
-  def enableLocalLimit: Boolean =
-    getBooleanConf("spark.auron.enable.local.limit", defaultValue = true)
-  def enableGlobalLimit: Boolean =
-    getBooleanConf("spark.auron.enable.global.limit", defaultValue = true)
+  def enableScan: Boolean = SparkAuronConfiguration.ENABLE_SCAN.get()
+  def enableProject: Boolean = SparkAuronConfiguration.ENABLE_PROJECT.get()
+  def enableFilter: Boolean = SparkAuronConfiguration.ENABLE_FILTER.get()
+  def enableSort: Boolean = SparkAuronConfiguration.ENABLE_SORT.get()
+  def enableUnion: Boolean = SparkAuronConfiguration.ENABLE_UNION.get()
+  def enableSmj: Boolean = SparkAuronConfiguration.ENABLE_SMJ.get()
+  def enableShj: Boolean = SparkAuronConfiguration.ENABLE_SHJ.get()
+  def enableBhj: Boolean = SparkAuronConfiguration.ENABLE_BHJ.get()
+  def enableBnlj: Boolean = SparkAuronConfiguration.ENABLE_BNLJ.get()
+  def enableLocalLimit: Boolean = SparkAuronConfiguration.ENABLE_LOCAL_LIMIT.get()
+  def enableGlobalLimit: Boolean = SparkAuronConfiguration.ENABLE_GLOBAL_LIMIT.get()
   def enableTakeOrderedAndProject: Boolean =
-    getBooleanConf("spark.auron.enable.take.ordered.and.project", defaultValue = true)
-  def enableAggr: Boolean =
-    getBooleanConf("spark.auron.enable.aggr", defaultValue = true)
-  def enableExpand: Boolean =
-    getBooleanConf("spark.auron.enable.expand", defaultValue = true)
-  def enableWindow: Boolean =
-    getBooleanConf("spark.auron.enable.window", defaultValue = true)
-  def enableWindowGroupLimit: Boolean =
-    getBooleanConf("spark.auron.enable.window.group.limit", defaultValue = true)
-  def enableGenerate: Boolean =
-    getBooleanConf("spark.auron.enable.generate", defaultValue = true)
-  def enableLocalTableScan: Boolean =
-    getBooleanConf("spark.auron.enable.local.table.scan", defaultValue = true)
-  def enableDataWriting: Boolean =
-    getBooleanConf("spark.auron.enable.data.writing", defaultValue = false)
-  def enableScanParquet: Boolean =
-    getBooleanConf("spark.auron.enable.scan.parquet", defaultValue = true)
-  def enableScanOrc: Boolean =
-    getBooleanConf("spark.auron.enable.scan.orc", defaultValue = true)
+    SparkAuronConfiguration.ENABLE_TAKE_ORDERED_AND_PROJECT.get()
+  def enableCollectLimit: Boolean = SparkAuronConfiguration.ENABLE_COLLECT_LIMIT.get()
+  def enableAggr: Boolean = SparkAuronConfiguration.ENABLE_AGGR.get()
+  def enableExpand: Boolean = SparkAuronConfiguration.ENABLE_EXPAND.get()
+  def enableWindow: Boolean = SparkAuronConfiguration.ENABLE_WINDOW.get()
+  def enableWindowGroupLimit: Boolean = SparkAuronConfiguration.ENABLE_WINDOW_GROUP_LIMIT.get()
+  def enableGenerate: Boolean = SparkAuronConfiguration.ENABLE_GENERATE.get()
+  def enableLocalTableScan: Boolean = SparkAuronConfiguration.ENABLE_LOCAL_TABLE_SCAN.get()
+  def enableDataWriting: Boolean = SparkAuronConfiguration.ENABLE_DATA_WRITING.get()
+  def enableScanParquet: Boolean = SparkAuronConfiguration.ENABLE_SCAN_PARQUET.get()
+  def enableScanParquetTimestamp: Boolean =
+    SparkAuronConfiguration.ENABLE_SCAN_PARQUET_TIMESTAMP.get()
+  def enableScanOrc: Boolean = SparkAuronConfiguration.ENABLE_SCAN_ORC.get()
+  def enableScanOrcTimestamp: Boolean = SparkAuronConfiguration.ENABLE_SCAN_ORC_TIMESTAMP.get()
+  def enableBroadcastExchange: Boolean = SparkAuronConfiguration.ENABLE_BROADCAST_EXCHANGE.get()
+  def enableShuffleExechange: Boolean = SparkAuronConfiguration.ENABLE_SHUFFLE_EXCHANGE.get()
 
   private val extConvertProviders = ServiceLoader.load(classOf[AuronConvertProvider]).asScala
+
+  private val sparkAuronConfig: AuronConfiguration =
+    AuronAdaptor.getInstance.getAuronConfiguration
+
   def extConvertSupported(exec: SparkPlan): Boolean = {
     extConvertProviders.exists(_.isSupported(exec))
   }
 
-  // format: off
-  // scalafix:off
-  // necessary imports for cross spark versions build
-  import org.apache.spark.sql.catalyst.plans._
-  import org.apache.spark.sql.catalyst.optimizer._
-  // scalafix:on
-  // format: on
+  def enableExchange: Boolean = {
+    enableShuffleExechange && supportedShuffleManager
+  }
+
+  private val supportedShuffleManagers: Seq[String] =
+    Seq("AuronShuffleManager", "AuronUniffleShuffleManager", "AuronCelebornShuffleManager")
+
+  def supportedShuffleManager: Boolean = {
+    val name = SQLConf.get.getConfString(config.SHUFFLE_MANAGER.key)
+    supportedShuffleManagers.exists(name.contains)
+  }
 
   def convertSparkPlanRecursively(exec: SparkPlan): SparkPlan = {
     // convert
@@ -177,8 +155,9 @@ object AuronConverters extends Logging {
 
   def convertSparkPlan(exec: SparkPlan): SparkPlan = {
     exec match {
-      case e: ShuffleExchangeExec => tryConvert(e, convertShuffleExchangeExec)
-      case e: BroadcastExchangeExec => tryConvert(e, convertBroadcastExchangeExec)
+      case e: ShuffleExchangeExec if enableExchange => tryConvert(e, convertShuffleExchangeExec)
+      case e: BroadcastExchangeExec if enableBroadcastExchange =>
+        tryConvert(e, convertBroadcastExchangeExec)
       case e: FileSourceScanExec if enableScan => // scan
         tryConvert(e, convertFileSourceScanExec)
       case e: ProjectExec if enableProject => // project
@@ -203,6 +182,8 @@ object AuronConverters extends Logging {
         tryConvert(e, convertGlobalLimitExec)
       case e: TakeOrderedAndProjectExec if enableTakeOrderedAndProject =>
         tryConvert(e, convertTakeOrderedAndProjectExec)
+      case e: CollectLimitExec if enableCollectLimit =>
+        tryConvert(e, convertCollectLimitExec)
 
       case e: HashAggregateExec if enableAggr => // hash aggregate
         val convertedAgg = tryConvert(e, convertHashAggregateExec)
@@ -265,16 +246,72 @@ object AuronConverters extends Logging {
                 if (Shims.get.isNative(exec)) { // for QueryStageInput and CustomShuffleReader
                   exec.setTagValue(convertibleTag, true)
                   exec.setTagValue(convertStrategyTag, AlwaysConvert)
+                  exec
                 } else {
-                  exec.setTagValue(convertibleTag, false)
-                  exec.setTagValue(convertStrategyTag, NeverConvert)
+                  addNeverConvertReasonTag(exec)
                 }
-                exec
             }
         }
     }
   }
 
+  private def addNeverConvertReasonTag(exec: SparkPlan) = {
+    val neverConvertReason =
+      exec match {
+        case _: FileSourceScanExec if !enableScan =>
+          "Conversion disabled: spark.auron.enable.scan=false."
+        case _: ProjectExec if !enableProject =>
+          "Conversion disabled: spark.auron.enable.project=false."
+        case _: FilterExec if !enableFilter =>
+          "Conversion disabled: spark.auron.enable.filter=false."
+        case _: SortExec if !enableSort =>
+          "Conversion disabled: spark.auron.enable.sort=false."
+        case _: UnionExec if !enableUnion =>
+          "Conversion disabled: spark.auron.enable.union=false."
+        case _: SortMergeJoinExec if !enableSmj =>
+          "Conversion disabled: spark.auron.enable.smj=false."
+        case _: ShuffledHashJoinExec if !enableShj =>
+          "Conversion disabled: spark.auron.enable.shj=false."
+        case _: BroadcastHashJoinExec if !enableBhj =>
+          "Conversion disabled: spark.auron.enable.bhj=false."
+        case _: BroadcastNestedLoopJoinExec if !enableBnlj =>
+          "Conversion disabled: spark.auron.enable.bnlj=false."
+        case _: LocalLimitExec if !enableLocalLimit =>
+          "Conversion disabled: spark.auron.enable.local.limit=false."
+        case _: GlobalLimitExec if !enableGlobalLimit =>
+          "Conversion disabled: spark.auron.enable.global.limit=false."
+        case _: TakeOrderedAndProjectExec if !enableTakeOrderedAndProject =>
+          "Conversion disabled: spark.auron.enable.take.ordered.and.project=false."
+        case _: CollectLimitExec if !enableCollectLimit =>
+          "Conversion disabled: spark.auron.enable.collectLimit=false."
+        case _: HashAggregateExec if !enableAggr =>
+          "Conversion disabled: spark.auron.enable.aggr=false."
+        case _: ObjectHashAggregateExec if !enableAggr =>
+          "Conversion disabled: spark.auron.enable.aggr=false."
+        case _: SortAggregateExec if !enableAggr =>
+          "Conversion disabled: spark.auron.enable.aggr=false."
+        case _: ExpandExec if !enableExpand =>
+          "Conversion disabled: spark.auron.enable.expand=false."
+        case _: WindowExec if !enableWindow =>
+          "Conversion disabled: spark.auron.enable.window=false."
+        case _: UnaryExecNode
+            if exec.getClass.getSimpleName == "WindowGroupLimitExec" && !enableWindowGroupLimit =>
+          "Conversion disabled: spark.auron.enable.window.group.limit=false."
+        case _: GenerateExec if !enableGenerate =>
+          "Conversion disabled: spark.auron.enable.generate=false."
+        case _: LocalTableScanExec if !enableLocalTableScan =>
+          "Conversion disabled: spark.auron.enable.local.table.scan=false."
+        case _: DataWritingCommandExec if !enableDataWriting =>
+          "Conversion disabled: spark.auron.enable.data.writing=false."
+        case _ =>
+          s"${exec.getClass.getSimpleName} is not supported yet."
+      }
+    exec.setTagValue(convertibleTag, false)
+    exec.setTagValue(convertStrategyTag, NeverConvert)
+    exec.setTagValue(neverConvertReasonTag, neverConvertReason)
+    exec
+  }
+
   def tryConvert[T <: SparkPlan](exec: T, convert: T => SparkPlan): SparkPlan = {
     try {
       exec.setTagValue(convertibleTag, true)
@@ -282,9 +319,31 @@ object AuronConverters extends Logging {
 
     } catch {
       case e @ (_: NotImplementedError | _: AssertionError | _: Exception) =>
-        logWarning(s"Falling back exec: ${exec.getClass.getSimpleName}: ${e.getMessage}")
+        if (log.isDebugEnabled()) {
+          logWarning(s"Falling back exec: ${exec.getClass.getSimpleName}: ${e.getMessage}", e)
+        } else {
+          logWarning(s"Falling back exec: ${exec.getClass.getSimpleName}: ${e.getMessage}")
+        }
+        val neverConvertReason = e match {
+          case _: AssertionError =>
+            exec match {
+              case _: FileSourceScanExec if enableScan =>
+                if (!enableScanParquet) {
+                  "Conversion disabled: spark.auron.enable.scan.parquet=false."
+                } else if (!enableScanOrc) {
+                  "Conversion disabled: spark.auron.enable.scan.orc=false."
+                } else {
+                  s"Falling back exec: ${exec.getClass.getSimpleName}: ${e.getMessage}"
+                }
+              case _ =>
+                s"Falling back exec: ${exec.getClass.getSimpleName}: ${e.getMessage}"
+            }
+          case _ =>
+            s"Falling back exec: ${exec.getClass.getSimpleName}: ${e.getMessage}"
+        }
         exec.setTagValue(convertibleTag, false)
         exec.setTagValue(convertStrategyTag, NeverConvert)
+        exec.setTagValue(neverConvertReasonTag, neverConvertReason)
         exec
     }
   }
@@ -303,7 +362,7 @@ object AuronConverters extends Logging {
     outputPartitioning match {
       case partitioning: RangePartitioning =>
         val unsupportedOrderType = partitioning.ordering
-          .find(e => !scalarTypeSupported(e.dataType))
+          .find(e => !isTypeSupported(e.dataType))
         assert(
           unsupportedOrderType.isEmpty,
           s"Unsupported order type in range partitioning: ${unsupportedOrderType.get}")
@@ -328,21 +387,9 @@ object AuronConverters extends Logging {
     Shims.get.createNativeShuffleExchangeExec(
       outputPartitioning,
       addRenameColumnsExec(convertedChild),
-      getShuffleOrigin(exec))
+      Shims.get.getShuffleOrigin(exec))
   }
 
-  @sparkver(" 3.2 / 3.3 / 3.4 / 3.5")
-  def getIsSkewJoinFromSHJ(exec: ShuffledHashJoinExec): Boolean = exec.isSkewJoin
-
-  @sparkver("3.0 / 3.1")
-  def getIsSkewJoinFromSHJ(exec: ShuffledHashJoinExec): Boolean = false
-
-  @sparkver("3.1 / 3.2 / 3.3 / 3.4 / 3.5")
-  def getShuffleOrigin(exec: ShuffleExchangeExec): Option[Any] = Some(exec.shuffleOrigin)
-
-  @sparkver("3.0")
-  def getShuffleOrigin(exec: ShuffleExchangeExec): Option[Any] = None
-
   def convertFileSourceScanExec(exec: FileSourceScanExec): SparkPlan = {
     val (
       relation,
@@ -373,9 +420,25 @@ object AuronConverters extends Logging {
     relation.fileFormat match {
       case p if p.getClass.getName.endsWith("ParquetFileFormat") =>
         assert(enableScanParquet)
+        if (!enableScanParquetTimestamp) {
+          assert(
+            !exec.requiredSchema.exists(e => existTimestampType(e.dataType)),
+            s"Parquet scan with timestamp type is not supported for table: ${tableIdentifier
+              .getOrElse("unknown")}. " +
+              "Set spark.auron.enable.scan.parquet.timestamp=true to enable timestamp support " +
+              "or remove timestamp columns from the query.")
+        }
         addRenameColumnsExec(Shims.get.createNativeParquetScanExec(exec))
       case p if p.getClass.getName.endsWith("OrcFileFormat") =>
         assert(enableScanOrc)
+        if (!enableScanOrcTimestamp) {
+          assert(
+            !exec.requiredSchema.exists(e => existTimestampType(e.dataType)),
+            s"ORC scan with timestamp type is not supported for tableIdentifier: ${tableIdentifier
+              .getOrElse("unknown")}. " +
+              "Set spark.auron.enable.scan.orc.timestamp=true to enable timestamp support " +
+              "or remove timestamp columns from the query.")
+        }
         addRenameColumnsExec(Shims.get.createNativeOrcScanExec(exec))
       case p =>
         throw new NotImplementedError(
@@ -420,7 +483,7 @@ object AuronConverters extends Logging {
 
     // force shuffled-hash join
     if (!requireOrdering
-      && AuronConf.FORCE_SHUFFLED_HASH_JOIN.booleanConf()
+      && sparkAuronConfig.getBoolean(SparkAuronConfiguration.FORCE_SHUFFLED_HASH_JOIN)
       && exec.children.forall(_.isInstanceOf[NativeSortBase])) {
       val (leftKeys, rightKeys, joinType, condition, left, right, isSkewJoin) =
         (
@@ -440,15 +503,14 @@ object AuronConverters extends Logging {
           "condition" -> condition))
       assert(condition.isEmpty, "join condition is not supported")
 
-      val buildSide = exec.getTagValue(joinSmallerSideTag) match {
-        case Some(org.apache.spark.sql.execution.auron.plan.BuildLeft) =>
-          org.apache.spark.sql.execution.auron.plan.BuildLeft
-        case Some(org.apache.spark.sql.execution.auron.plan.BuildRight) =>
-          org.apache.spark.sql.execution.auron.plan.BuildRight
-        case None =>
+      val buildSide = exec
+        .getTagValue(joinSmallerSideTag)
+        .map(_.asInstanceOf[JoinBuildSide])
+        .getOrElse {
           logWarning("JoinSmallerSideTag is missing, defaults to BuildRight")
-          org.apache.spark.sql.execution.auron.plan.BuildRight
-      }
+          JoinBuildRight
+        }
+
       return Shims.get.createNativeShuffledHashJoinExec(
         addRenameColumnsExec(convertToNative(left.children(0))),
         addRenameColumnsExec(convertToNative(right.children(0))),
@@ -487,14 +549,9 @@ object AuronConverters extends Logging {
   }
 
   def convertShuffledHashJoinExec(exec: ShuffledHashJoinExec): SparkPlan = {
-    val (leftKeys, rightKeys, joinType, condition, left, right, buildSide) = (
-      exec.leftKeys,
-      exec.rightKeys,
-      exec.joinType,
-      exec.condition,
-      exec.left,
-      exec.right,
-      exec.buildSide)
+    val buildSide = Shims.get.getJoinBuildSide(exec)
+    val (leftKeys, rightKeys, joinType, condition, left, right) =
+      (exec.leftKeys, exec.rightKeys, exec.joinType, exec.condition, exec.left, exec.right)
     logDebugPlanConversion(
       exec,
       Seq(
@@ -511,14 +568,10 @@ object AuronConverters extends Logging {
         leftKeys,
         rightKeys,
         joinType,
-        buildSide match {
-          case BuildLeft => org.apache.spark.sql.execution.auron.plan.BuildLeft
-          case BuildRight => org.apache.spark.sql.execution.auron.plan.BuildRight
-        },
-        getIsSkewJoinFromSHJ(exec))
-
+        buildSide,
+        Shims.get.getIsSkewJoinFromSHJ(exec))
     } catch {
-      case _ if AuronConf.FORCE_SHUFFLED_HASH_JOIN.booleanConf() =>
+      case _ if sparkAuronConfig.getBoolean(SparkAuronConfiguration.FORCE_SHUFFLED_HASH_JOIN) =>
         logWarning(
           "in forceShuffledHashJoin mode, hash joins are likely too run OOM because of " +
             "small on-heap memory configuration. to avoid this, we will fall back this " +
@@ -556,15 +609,17 @@ object AuronConverters extends Logging {
   }
 
   def convertBroadcastHashJoinExec(exec: BroadcastHashJoinExec): SparkPlan = {
+    val buildSide = Shims.get.getJoinBuildSide(exec)
     try {
-      val (leftKeys, rightKeys, joinType, buildSide, condition, left, right) = (
-        exec.leftKeys,
-        exec.rightKeys,
-        exec.joinType,
-        exec.buildSide,
-        exec.condition,
-        exec.left,
-        exec.right)
+      val (leftKeys, rightKeys, joinType, condition, left, right, naaj) =
+        (
+          exec.leftKeys,
+          exec.rightKeys,
+          exec.joinType,
+          exec.condition,
+          exec.left,
+          exec.right,
+          Shims.get.isNullAwareAntiJoin(exec))
       logDebugPlanConversion(
         exec,
         Seq(
@@ -577,9 +632,9 @@ object AuronConverters extends Logging {
 
       // verify build side is native
       buildSide match {
-        case BuildRight =>
+        case JoinBuildRight =>
           assert(NativeHelper.isNative(right), "broadcast join build side is not native")
-        case BuildLeft =>
+        case JoinBuildLeft =>
           assert(NativeHelper.isNative(left), "broadcast join build side is not native")
       }
 
@@ -590,16 +645,14 @@ object AuronConverters extends Logging {
         leftKeys,
         rightKeys,
         joinType,
-        buildSide match {
-          case BuildLeft => BroadcastLeft
-          case BuildRight => BroadcastRight
-        })
+        buildSide,
+        naaj)
 
     } catch {
       case e @ (_: NotImplementedError | _: Exception) =>
-        val underlyingBroadcast = exec.buildSide match {
-          case BuildLeft => Shims.get.getUnderlyingBroadcast(exec.left)
-          case BuildRight => Shims.get.getUnderlyingBroadcast(exec.right)
+        val underlyingBroadcast = buildSide match {
+          case JoinBuildLeft => Shims.get.getUnderlyingBroadcast(exec.left)
+          case JoinBuildRight => Shims.get.getUnderlyingBroadcast(exec.right)
         }
         underlyingBroadcast.setTagValue(NativeBroadcastExchangeBase.nativeExecutionTag, false)
         throw e
@@ -607,9 +660,10 @@ object AuronConverters extends Logging {
   }
 
   def convertBroadcastNestedLoopJoinExec(exec: BroadcastNestedLoopJoinExec): SparkPlan = {
+    val buildSide = Shims.get.getJoinBuildSide(exec)
     try {
-      val (joinType, buildSide, condition, left, right) =
-        (exec.joinType, exec.buildSide, exec.condition, exec.left, exec.right)
+      val (joinType, condition, left, right) =
+        (exec.joinType, exec.condition, exec.left, exec.right)
       logDebugPlanConversion(
         exec,
         Seq("joinType" -> joinType, "condition" -> condition, "buildSide" -> buildSide))
@@ -618,9 +672,9 @@ object AuronConverters extends Logging {
 
       // verify build side is native
       buildSide match {
-        case BuildRight =>
+        case JoinBuildRight =>
           assert(NativeHelper.isNative(right), "broadcast join build side is not native")
-        case BuildLeft =>
+        case JoinBuildLeft =>
           assert(NativeHelper.isNative(left), "broadcast join build side is not native")
       }
 
@@ -632,47 +686,43 @@ object AuronConverters extends Logging {
         Nil,
         Nil,
         joinType,
-        buildSide match {
-          case BuildLeft => BroadcastLeft
-          case BuildRight => BroadcastRight
-        })
-
+        buildSide,
+        isNullAwareAntiJoin = false)
     } catch {
       case e @ (_: NotImplementedError | _: Exception) =>
-        val underlyingBroadcast = exec.buildSide match {
-          case BuildLeft => Shims.get.getUnderlyingBroadcast(exec.left)
-          case BuildRight => Shims.get.getUnderlyingBroadcast(exec.right)
+        val underlyingBroadcast = buildSide match {
+          case JoinBuildLeft => Shims.get.getUnderlyingBroadcast(exec.left)
+          case JoinBuildRight => Shims.get.getUnderlyingBroadcast(exec.right)
         }
         underlyingBroadcast.setTagValue(NativeBroadcastExchangeBase.nativeExecutionTag, false)
         throw e
     }
   }
 
-  def convertBroadcastExchangeExec(exec: SparkPlan): SparkPlan = {
-    exec match {
-      case exec: BroadcastExchangeExec =>
-        logDebugPlanConversion(exec, Seq("mode" -> exec.mode))
-        val converted = Shims.get.createNativeBroadcastExchangeExec(exec.mode, exec.child)
-        converted.setTagValue(NativeBroadcastExchangeBase.nativeExecutionTag, true)
-        return converted
-    }
-    exec
+  def convertBroadcastExchangeExec(exec: BroadcastExchangeExec): SparkPlan = {
+    logDebugPlanConversion(exec, Seq("mode" -> exec.mode))
+    val converted = Shims.get.createNativeBroadcastExchangeExec(exec.mode, exec.child)
+    converted.setTagValue(NativeBroadcastExchangeBase.nativeExecutionTag, true)
+    converted
   }
 
   def convertLocalLimitExec(exec: LocalLimitExec): SparkPlan = {
     logDebugPlanConversion(exec)
-    Shims.get.createNativeLocalLimitExec(exec.limit.toLong, exec.child)
+    Shims.get.createNativeLocalLimitExec(exec.limit, exec.child)
   }
 
   def convertGlobalLimitExec(exec: GlobalLimitExec): SparkPlan = {
     logDebugPlanConversion(exec)
-    Shims.get.createNativeGlobalLimitExec(exec.limit.toLong, exec.child)
+    val (limit, offset) = Shims.get.getLimitAndOffset(exec)
+    Shims.get.createNativeGlobalLimitExec(limit, offset, exec.child)
   }
 
   def convertTakeOrderedAndProjectExec(exec: TakeOrderedAndProjectExec): SparkPlan = {
     logDebugPlanConversion(exec)
+    val (limit, offset) = Shims.get.getLimitAndOffset(exec)
     val nativeTakeOrdered = Shims.get.createNativeTakeOrderedExec(
-      exec.limit,
+      limit,
+      offset,
       exec.sortOrder,
       addRenameColumnsExec(convertToNative(exec.child)))
 
@@ -684,6 +734,12 @@ object AuronConverters extends Logging {
     }
   }
 
+  def convertCollectLimitExec(exec: CollectLimitExec): SparkPlan = {
+    logDebugPlanConversion(exec)
+    val (limit, offset) = Shims.get.getLimitAndOffset(exec)
+    Shims.get.createNativeCollectLimitExec(limit, offset, exec.child)
+  }
+
   def convertHashAggregateExec(exec: HashAggregateExec): SparkPlan = {
     // split non-trivial children exprs in partial-agg to a ProjectExec
     // for enabling filter-project optimization in native side
@@ -740,8 +796,7 @@ object AuronConverters extends Logging {
           addRenameColumnsExec(convertToNative(exec.child))
         case _ =>
           if (needRenameColumns(exec.child)) {
-            val newNames = exec.groupingExpressions.map(Util.getFieldNameByExprId) :+
-              NativeAggBase.AGG_BUF_COLUMN_NAME
+            val newNames = exec.groupingExpressions.map(Util.getFieldNameByExprId)
             Shims.get.createNativeRenameColumnsExec(convertToNative(exec.child), newNames)
           } else {
             convertToNative(exec.child)
@@ -750,7 +805,7 @@ object AuronConverters extends Logging {
 
     val isFinal = exec.requiredChildDistributionExpressions.isDefined &&
       exec.aggregateExpressions.forall(_.mode == Final)
-    if (isFinal) { // wraps with a projection to handle resutExpressions
+    if (isFinal) { // wraps with a projection to handle resultExpressions
       try {
         return Shims.get.createNativeProjectExec(exec.resultExpressions, nativeAggr)
       } catch {
@@ -797,8 +852,7 @@ object AuronConverters extends Logging {
           addRenameColumnsExec(convertToNative(exec.child))
         case _ =>
           if (needRenameColumns(exec.child)) {
-            val newNames = exec.groupingExpressions.map(Util.getFieldNameByExprId) :+
-              NativeAggBase.AGG_BUF_COLUMN_NAME
+            val newNames = exec.groupingExpressions.map(Util.getFieldNameByExprId)
             Shims.get.createNativeRenameColumnsExec(convertToNative(exec.child), newNames)
           } else {
             convertToNative(exec.child)
@@ -807,7 +861,7 @@ object AuronConverters extends Logging {
 
     val isFinal = exec.requiredChildDistributionExpressions.isDefined &&
       exec.aggregateExpressions.forall(_.mode == Final)
-    if (isFinal) { // wraps with a projection to handle resutExpressions
+    if (isFinal) { // wraps with a projection to handle resultExpressions
       try {
         return Shims.get.createNativeProjectExec(exec.resultExpressions, nativeAggr)
       } catch {
@@ -851,8 +905,7 @@ object AuronConverters extends Logging {
           addRenameColumnsExec(convertToNative(child))
         case _ =>
           if (needRenameColumns(child)) {
-            val newNames = exec.groupingExpressions.map(Util.getFieldNameByExprId) :+
-              NativeAggBase.AGG_BUF_COLUMN_NAME
+            val newNames = exec.groupingExpressions.map(Util.getFieldNameByExprId)
             Shims.get.createNativeRenameColumnsExec(convertToNative(child), newNames)
           } else {
             convertToNative(child)
@@ -861,7 +914,7 @@ object AuronConverters extends Logging {
 
     val isFinal = exec.requiredChildDistributionExpressions.isDefined &&
       exec.aggregateExpressions.forall(_.mode == Final)
-    if (isFinal) { // wraps with a projection to handle resutExpressions
+    if (isFinal) { // wraps with a projection to handle resultExpressions
       try {
         return Shims.get.createNativeProjectExec(exec.resultExpressions, nativeAggr)
       } catch {
@@ -1000,8 +1053,12 @@ object AuronConverters extends Logging {
         extends LeafExecNode
         with NativeSupports {
 
+      private def nativeSchema = Util.getNativeSchema(output)
+
+      // check whether native converting is supported
+      nativeSchema
+
       override protected def doExecuteNative(): NativeRDD = {
-        val nativeSchema = Util.getNativeSchema(output)
         val partitions = Range(0, outputPartitioning.numPartitions)
           .map(i =>
             new Partition {
@@ -1011,12 +1068,12 @@ object AuronConverters extends Logging {
 
         new NativeRDD(
           sparkContext,
-          MetricNode(Map(), Nil),
+          SparkMetricNode(Map(), Nil),
           rddPartitions = partitions,
           rddPartitioner = None,
           rddDependencies = Nil,
           false,
-          (_partition, _taskContext) => {
+          (_, _) => {
             val nativeEmptyExec = EmptyPartitionsExecNode
               .newBuilder()
               .setNumPartitions(outputPartitioning.numPartitions)
@@ -1025,6 +1082,8 @@ object AuronConverters extends Logging {
           },
           friendlyName = "NativeRDD.Empty")
       }
+
+      override val nodeName: String = s"NativeEmpty"
     }
     NativeEmptyExec(output, outputPartitioning, outputOrdering)
   }
