@@ -32,6 +32,7 @@ import org.apache.arrow.vector.VectorSchemaRoot
 import org.apache.arrow.vector.ipc.ArrowStreamWriter
 import org.apache.commons.lang3.reflect.FieldUtils
 import org.apache.commons.lang3.reflect.MethodUtils
+import org.apache.spark.TaskContext
 import org.apache.spark.internal.Logging
 import org.apache.spark.sql.auron.util.Using
 import org.apache.spark.sql.catalyst.InternalRow
@@ -81,16 +82,21 @@ import org.apache.spark.util.Utils
 
 import org.apache.auron.{protobuf => pb}
 import org.apache.auron.protobuf.PhysicalExprNode
+import org.apache.auron.spark.configuration.SparkAuronConfiguration
 
 object NativeConverters extends Logging {
-  def udfJsonEnabled: Boolean =
-    AuronConverters.getBooleanConf("spark.auron.udf.UDFJson.enabled", defaultValue = true)
-  def udfBrickHouseEnabled: Boolean =
-    AuronConverters.getBooleanConf("spark.auron.udf.brickhouse.enabled", defaultValue = true)
-  def decimalArithOpEnabled: Boolean =
-    AuronConverters.getBooleanConf("spark.auron.decimal.arithOp.enabled", defaultValue = false)
+  def udfJsonEnabled: Boolean = SparkAuronConfiguration.UDF_JSON_ENABLED.get()
+  def udfBrickHouseEnabled: Boolean = SparkAuronConfiguration.UDF_BRICKHOUSE_ENABLED.get()
+  def decimalArithOpEnabled: Boolean = SparkAuronConfiguration.DECIMAL_ARITH_OP_ENABLED.get()
+  def datetimeExtractEnabled: Boolean = SparkAuronConfiguration.DATETIME_EXTRACT_ENABLED.get()
+  def castTrimStringEnabled: Boolean = SparkAuronConfiguration.CAST_STRING_TRIM_ENABLE.get()
+  def singleChildFallbackEnabled: Boolean =
+    SparkAuronConfiguration.UDF_SINGLE_CHILD_FALLBACK_ENABLED.get()
 
-  def scalarTypeSupported(dataType: DataType): Boolean = {
+  /**
+   * Is the data type(scalar or complex) supported by Auron.
+   */
+  def isTypeSupported(dataType: DataType): Boolean = {
     dataType match {
       case NullType | BooleanType | ByteType | ShortType | IntegerType | LongType | FloatType |
           DoubleType | StringType | DateType | TimestampType =>
@@ -99,11 +105,24 @@ object NativeConverters extends Logging {
       case _: DecimalType =>
         // auron only supports 64-bit decimal
         false
-      case at: ArrayType => scalarTypeSupported(at.elementType)
+      case at: ArrayType => isTypeSupported(at.elementType)
       case m: MapType =>
-        scalarTypeSupported(m.keyType) && scalarTypeSupported(m.valueType)
+        isTypeSupported(m.keyType) && isTypeSupported(m.valueType)
       case s: StructType =>
-        s.fields.forall(e => scalarTypeSupported(e.dataType))
+        s.fields.forall(e => isTypeSupported(e.dataType))
+      case _ => false
+    }
+  }
+
+  def existTimestampType(dataType: DataType): Boolean = {
+    dataType match {
+      case TimestampType =>
+        true
+      case at: ArrayType => existTimestampType(at.elementType)
+      case m: MapType =>
+        existTimestampType(m.keyType) || existTimestampType(m.valueType)
+      case s: StructType =>
+        s.fields.exists(e => existTimestampType(e.dataType))
       case _ => false
     }
   }
@@ -260,6 +279,10 @@ object NativeConverters extends Logging {
       throw new NotImplementedError(s"unsupported expression: (${e.getClass}) $e")
     }
 
+    if (!singleChildFallbackEnabled) {
+      return convertExprWithFallback(sparkExpr, isPruningExpr = false, fallbackToError)
+    }
+
     try {
       // get number of inconvertible children
       var numInconvertibleChildren = 0
@@ -452,7 +475,7 @@ object NativeConverters extends Logging {
           if (cast.child.dataType == StringType &&
             (cast.dataType.isInstanceOf[NumericType] || cast.dataType
               .isInstanceOf[BooleanType]) &&
-            AuronConf.CAST_STRING_TRIM_ENABLE.booleanConf()) {
+            castTrimStringEnabled) {
             // converting Cast(str as num) to StringTrim(Cast(str as num)) if enabled
             StringTrim(cast.child)
           } else {
@@ -528,6 +551,7 @@ object NativeConverters extends Logging {
       case LessThan(lhs, rhs) => buildBinaryExprNode(lhs, rhs, "Lt")
       case GreaterThanOrEqual(lhs, rhs) => buildBinaryExprNode(lhs, rhs, "GtEq")
       case LessThanOrEqual(lhs, rhs) => buildBinaryExprNode(lhs, rhs, "LtEq")
+      case EqualNullSafe(lhs, rhs) => buildBinaryExprNode(lhs, rhs, "IsNotDistinctFrom")
 
       case e: Add if !e.dataType.isInstanceOf[DecimalType] || decimalArithOpEnabled =>
         val lhs = e.left
@@ -688,7 +712,7 @@ object NativeConverters extends Logging {
                   pb.PhysicalBinaryExprNode
                     .newBuilder()
                     .setL(convertExprWithFallback(Cast(lhs, resultType), isPruningExpr, fallback))
-                    .setR(buildExtScalarFunction("NullIfZero", rhs :: Nil, rhs.dataType))
+                    .setR(buildExtScalarFunction("Spark_NullIfZero", rhs :: Nil, rhs.dataType))
                     .setOp("Divide"))
               }))
           }
@@ -701,7 +725,7 @@ object NativeConverters extends Logging {
               pb.PhysicalBinaryExprNode
                 .newBuilder()
                 .setL(convertExprWithFallback(lhsCasted, isPruningExpr, fallback))
-                .setR(buildExtScalarFunction("NullIfZero", rhsCasted :: Nil, rhs.dataType))
+                .setR(buildExtScalarFunction("Spark_NullIfZero", rhsCasted :: Nil, rhs.dataType))
                 .setOp("Divide"))
           }
         }
@@ -723,7 +747,8 @@ object NativeConverters extends Logging {
                 pb.PhysicalBinaryExprNode
                   .newBuilder()
                   .setL(convertExprWithFallback(lhsCasted, isPruningExpr, fallback))
-                  .setR(buildExtScalarFunction("NullIfZero", rhsCasted :: Nil, rhs.dataType))
+                  .setR(
+                    buildExtScalarFunction("Spark_NullIfZero", rhsCasted :: Nil, rhs.dataType))
                   .setOp("Modulo"))
             }
         }
@@ -740,7 +765,10 @@ object NativeConverters extends Logging {
         }
 
       // if rhs is complex in and/or operators, use short-circuiting implementation
-      case And(lhs, rhs) if rhs.find(HiveUDFUtil.isHiveUDF).isDefined =>
+      // or if forceShortCircuitAndOr is enabled, always use short-circuiting
+      case And(lhs, rhs)
+          if SparkAuronConfiguration.FORCE_SHORT_CIRCUIT_AND_OR.get()
+            || rhs.find(HiveUDFUtil.isHiveUDF).isDefined =>
         buildExprNode {
           _.setScAndExpr(
             pb.PhysicalSCAndExprNode
@@ -748,7 +776,9 @@ object NativeConverters extends Logging {
               .setLeft(convertExprWithFallback(lhs, isPruningExpr, fallback))
               .setRight(convertExprWithFallback(rhs, isPruningExpr, fallback)))
         }
-      case Or(lhs, rhs) if rhs.find(HiveUDFUtil.isHiveUDF).isDefined =>
+      case Or(lhs, rhs)
+          if SparkAuronConfiguration.FORCE_SHORT_CIRCUIT_AND_OR.get()
+            || rhs.find(HiveUDFUtil.isHiveUDF).isDefined =>
         buildExprNode {
           _.setScOrExpr(
             pb.PhysicalSCOrExprNode
@@ -784,6 +814,8 @@ object NativeConverters extends Logging {
         buildScalarFunction(pb.ScalarFunction.Log2, e.children.map(nullIfNegative), e.dataType)
       case e: Log10 =>
         buildScalarFunction(pb.ScalarFunction.Log10, e.children.map(nullIfNegative), e.dataType)
+      case e: Nvl2 =>
+        buildScalarFunction(pb.ScalarFunction.Nvl2, e.children, e.dataType)
       case e: Floor if !e.dataType.isInstanceOf[DecimalType] =>
         if (e.child.dataType.isInstanceOf[LongType]) {
           convertExprWithFallback(e.child, isPruningExpr, fallback)
@@ -811,15 +843,31 @@ object NativeConverters extends Logging {
           }
         }
       case e: Expm1 => buildScalarFunction(pb.ScalarFunction.Expm1, e.children, e.dataType)
+      case e: Least => buildScalarFunction(pb.ScalarFunction.Least, e.children, e.dataType)
       case e: Factorial =>
         buildScalarFunction(pb.ScalarFunction.Factorial, e.children, e.dataType)
       case e: Hex => buildScalarFunction(pb.ScalarFunction.Hex, e.children, e.dataType)
+      case e: IsNaN =>
+        buildExtScalarFunction("Spark_IsNaN", e.children, e.dataType)
+      case e: Round =>
+        e.scale match {
+          case Literal(n: Int, _) =>
+            buildExtScalarFunction("Spark_Round", Seq(e.child, Literal(n.toLong)), e.dataType)
+          case _ =>
+            buildExtScalarFunction("Spark_Round", Seq(e.child, Literal(0L)), e.dataType)
+        }
 
-      // TODO: datafusion's round() has different behavior from spark
-      // case e @ Round(_1, Literal(n: Int, _)) if _1.dataType.isInstanceOf[FractionalType] =>
-      //   buildScalarFunction(pb.ScalarFunction.Round, Seq(_1, Literal(n.toLong)), e.dataType)
+      case e: BRound =>
+        e.scale match {
+          case Literal(n: Int, _) =>
+            buildExtScalarFunction("Spark_BRound", Seq(e.child, Literal(n.toLong)), e.dataType)
+          case _ =>
+            buildExtScalarFunction("Spark_BRound", Seq(e.child, Literal(0L)), e.dataType)
+        }
 
       case e: Signum => buildScalarFunction(pb.ScalarFunction.Signum, e.children, e.dataType)
+      case e: FindInSet =>
+        buildScalarFunction(pb.ScalarFunction.FindInSet, e.children, e.dataType)
       case e: Abs if e.dataType.isInstanceOf[FloatType] || e.dataType.isInstanceOf[DoubleType] =>
         buildScalarFunction(pb.ScalarFunction.Abs, e.children, e.dataType)
       case e: OctetLength =>
@@ -827,10 +875,10 @@ object NativeConverters extends Logging {
       case Length(arg) if arg.dataType == StringType =>
         buildScalarFunction(pb.ScalarFunction.CharacterLength, arg :: Nil, IntegerType)
 
-      case e: Lower if AuronConf.CASE_CONVERT_FUNCTIONS_ENABLE.booleanConf() =>
-        buildExtScalarFunction("StringLower", e.children, e.dataType)
-      case e: Upper if AuronConf.CASE_CONVERT_FUNCTIONS_ENABLE.booleanConf() =>
-        buildExtScalarFunction("StringUpper", e.children, e.dataType)
+      case e: Lower if SparkAuronConfiguration.CASE_CONVERT_FUNCTIONS_ENABLE.get() =>
+        buildExtScalarFunction("Spark_StringLower", e.children, e.dataType)
+      case e: Upper if SparkAuronConfiguration.CASE_CONVERT_FUNCTIONS_ENABLE.get() =>
+        buildExtScalarFunction("Spark_StringUpper", e.children, e.dataType)
 
       case e: StringTrim =>
         buildScalarFunction(pb.ScalarFunction.Trim, e.srcStr +: e.trimStr.toSeq, e.dataType)
@@ -839,27 +887,48 @@ object NativeConverters extends Logging {
       case e: StringTrimRight =>
         buildScalarFunction(pb.ScalarFunction.Rtrim, e.srcStr +: e.trimStr.toSeq, e.dataType)
       case e @ NullIf(left, right, _) =>
-        buildExtScalarFunction("NullIf", left :: right :: Nil, e.dataType)
+        buildExtScalarFunction("Spark_NullIf", left :: right :: Nil, e.dataType)
       case Md5(_1) =>
-        buildScalarFunction(pb.ScalarFunction.MD5, Seq(unpackBinaryTypeCast(_1)), StringType)
+        buildExtScalarFunction("Spark_MD5", Seq(unpackBinaryTypeCast(_1)), StringType)
+      case Reverse(_1) =>
+        buildScalarFunction(pb.ScalarFunction.Reverse, Seq(unpackBinaryTypeCast(_1)), StringType)
+      case e: InitCap =>
+        buildExtScalarFunction("Spark_InitCap", e.children, e.dataType)
       case Sha2(_1, Literal(224, _)) =>
-        buildExtScalarFunction("Sha224", Seq(unpackBinaryTypeCast(_1)), StringType)
+        buildExtScalarFunction("Spark_Sha224", Seq(unpackBinaryTypeCast(_1)), StringType)
       case Sha2(_1, Literal(0, _)) =>
-        buildExtScalarFunction("Sha256", Seq(unpackBinaryTypeCast(_1)), StringType)
+        buildExtScalarFunction("Spark_Sha256", Seq(unpackBinaryTypeCast(_1)), StringType)
       case Sha2(_1, Literal(256, _)) =>
-        buildExtScalarFunction("Sha256", Seq(unpackBinaryTypeCast(_1)), StringType)
+        buildExtScalarFunction("Spark_Sha256", Seq(unpackBinaryTypeCast(_1)), StringType)
       case Sha2(_1, Literal(384, _)) =>
-        buildExtScalarFunction("Sha384", Seq(unpackBinaryTypeCast(_1)), StringType)
+        buildExtScalarFunction("Spark_Sha384", Seq(unpackBinaryTypeCast(_1)), StringType)
       case Sha2(_1, Literal(512, _)) =>
-        buildExtScalarFunction("Sha512", Seq(unpackBinaryTypeCast(_1)), StringType)
+        buildExtScalarFunction("Spark_Sha512", Seq(unpackBinaryTypeCast(_1)), StringType)
       case Murmur3Hash(children, 42) =>
-        buildExtScalarFunction("Murmur3Hash", children, IntegerType)
+        buildExtScalarFunction("Spark_Murmur3Hash", children, IntegerType)
       case XxHash64(children, 42L) =>
-        buildExtScalarFunction("XxHash64", children, LongType)
+        buildExtScalarFunction("Spark_XxHash64", children, LongType)
+      case e: Greatest =>
+        buildScalarFunction(pb.ScalarFunction.Greatest, e.children, e.dataType)
+      case e: Pow =>
+        buildScalarFunction(pb.ScalarFunction.Power, e.children, e.dataType)
+      case e: Nvl =>
+        buildScalarFunction(pb.ScalarFunction.Nvl, e.children, e.dataType)
 
-      case Year(child) => buildExtScalarFunction("Year", child :: Nil, IntegerType)
-      case Month(child) => buildExtScalarFunction("Month", child :: Nil, IntegerType)
-      case DayOfMonth(child) => buildExtScalarFunction("Day", child :: Nil, IntegerType)
+      case Year(child) => buildExtScalarFunction("Spark_Year", child :: Nil, IntegerType)
+      case Month(child) => buildExtScalarFunction("Spark_Month", child :: Nil, IntegerType)
+      case DayOfMonth(child) => buildExtScalarFunction("Spark_Day", child :: Nil, IntegerType)
+      case Quarter(child) => buildExtScalarFunction("Spark_Quarter", child :: Nil, IntegerType)
+
+      case e: Levenshtein =>
+        buildScalarFunction(pb.ScalarFunction.Levenshtein, e.children, e.dataType)
+
+      case e: Hour if datetimeExtractEnabled =>
+        buildTimePartExt("Spark_Hour", e.children.head, isPruningExpr, fallback)
+      case e: Minute if datetimeExtractEnabled =>
+        buildTimePartExt("Spark_Minute", e.children.head, isPruningExpr, fallback)
+      case e: Second if datetimeExtractEnabled =>
+        buildTimePartExt("Spark_Second", e.children.head, isPruningExpr, fallback)
 
       // startswith is converted to scalar function in pruning-expr mode
       case StartsWith(expr, Literal(prefix, StringType)) if isPruningExpr =>
@@ -906,25 +975,36 @@ object NativeConverters extends Logging {
           StringType)
 
       case StringSpace(n) =>
-        buildExtScalarFunction("StringSpace", n :: Nil, StringType)
+        buildExtScalarFunction("Spark_StringSpace", n :: Nil, StringType)
 
       case StringRepeat(str, n @ Literal(_, IntegerType)) =>
-        buildExtScalarFunction("StringRepeat", str :: n :: Nil, StringType)
+        buildExtScalarFunction("Spark_StringRepeat", str :: n :: Nil, StringType)
 
       case e: Concat if e.children.forall(_.dataType == StringType) =>
-        buildExtScalarFunction("StringConcat", e.children, e.dataType)
+        buildExtScalarFunction("Spark_StringConcat", e.children, e.dataType)
 
       case e: ConcatWs
           if e.children.nonEmpty
             && e.children.head.isInstanceOf[Literal]
             && e.children.forall(c =>
               c.dataType == StringType || c.dataType == ArrayType(StringType)) =>
-        buildExtScalarFunction("StringConcatWs", e.children, e.dataType)
+        buildExtScalarFunction("Spark_StringConcatWs", e.children, e.dataType)
 
       case e: Coalesce =>
         val children = e.children.map(Cast(_, e.dataType))
         buildScalarFunction(pb.ScalarFunction.Coalesce, children, e.dataType)
 
+      case _ @StringLPad(str, len, pad) =>
+        buildScalarFunction(
+          pb.ScalarFunction.Lpad,
+          Seq(str, castIfNecessary(len, LongType), pad),
+          StringType)
+      case _ @StringRPad(str, len, pad) =>
+        buildScalarFunction(
+          pb.ScalarFunction.Rpad,
+          Seq(str, castIfNecessary(len, LongType), pad),
+          StringType)
+
       case e @ If(predicate, trueValue, falseValue) =>
         val castedTrueValue = trueValue match {
           case t if t.dataType != e.dataType => Cast(t, e.dataType)
@@ -963,7 +1043,7 @@ object NativeConverters extends Logging {
       // expressions for DecimalPrecision rule
       case UnscaledValue(_1) if decimalArithOpEnabled =>
         val args = _1 :: Nil
-        buildExtScalarFunction("UnscaledValue", args, LongType)
+        buildExtScalarFunction("Spark_UnscaledValue", args, LongType)
 
       case e: MakeDecimal if decimalArithOpEnabled =>
         val precision = e.precision
@@ -971,7 +1051,7 @@ object NativeConverters extends Logging {
         val args =
           e.child :: Literal
             .apply(precision, IntegerType) :: Literal.apply(scale, IntegerType) :: Nil
-        buildExtScalarFunction("MakeDecimal", args, DecimalType(precision, scale))
+        buildExtScalarFunction("Spark_MakeDecimal", args, DecimalType(precision, scale))
 
       case e: CheckOverflow if decimalArithOpEnabled =>
         val precision = e.dataType.precision
@@ -979,13 +1059,13 @@ object NativeConverters extends Logging {
         val args =
           e.child :: Literal
             .apply(precision, IntegerType) :: Literal.apply(scale, IntegerType) :: Nil
-        buildExtScalarFunction("CheckOverflow", args, DecimalType(precision, scale))
+        buildExtScalarFunction("Spark_CheckOverflow", args, DecimalType(precision, scale))
 
       case e: NormalizeNaNAndZero
           if e.dataType.isInstanceOf[FloatType] || e.dataType.isInstanceOf[DoubleType] =>
-        buildExtScalarFunction("NormalizeNanAndZero", e.children, e.dataType)
+        buildExtScalarFunction("Spark_NormalizeNanAndZero", e.children, e.dataType)
 
-      case e: CreateArray => buildExtScalarFunction("MakeArray", e.children, e.dataType)
+      case e: CreateArray => buildExtScalarFunction("Spark_MakeArray", e.children, e.dataType)
 
       case e: CreateNamedStruct =>
         buildExprNode {
@@ -1038,6 +1118,11 @@ object NativeConverters extends Logging {
           _.setRowNumExpr(pb.RowNumExprNode.newBuilder())
         }
 
+      case StubExpr("SparkPartitionID", _, _) =>
+        buildExprNode {
+          _.setSparkPartitionIdExpr(pb.SparkPartitionIdExprNode.newBuilder())
+        }
+
       // hive UDFJson
       case e
           if udfJsonEnabled && (
@@ -1052,16 +1137,19 @@ object NativeConverters extends Logging {
         // The benefit of this approach is that if there are multiple calls,
         // the JSON object can be reused, which can significantly improve performance.
         val parsed = Shims.get.createNativeExprWrapper(
-          buildExtScalarFunction("ParseJson", e.children(0) :: Nil, BinaryType),
+          buildExtScalarFunction("Spark_ParseJson", e.children(0) :: Nil, BinaryType),
           BinaryType,
           nullable = false)
-        buildExtScalarFunction("GetParsedJsonObject", parsed :: e.children(1) :: Nil, StringType)
+        buildExtScalarFunction(
+          "Spark_GetParsedJsonObject",
+          parsed :: e.children(1) :: Nil,
+          StringType)
 
       // hive UDF brickhouse.array_union
       case e
           if getFunctionClassName(e).contains("brickhouse.udf.collect.ArrayUnionUDF")
             && udfBrickHouseEnabled =>
-        buildExtScalarFunction("BrickhouseArrayUnion", e.children, e.dataType)
+        buildExtScalarFunction("Spark_BrickhouseArrayUnion", e.children, e.dataType)
 
       case e =>
         Shims.get.convertMoreExprWithFallback(e, isPruningExpr, fallback) match {
@@ -1150,7 +1238,7 @@ object NativeConverters extends Logging {
         }
 
         // fallback to UDAF
-        if (AuronConf.UDAF_FALLBACK_ENABLE.booleanConf()) {
+        if (SparkAuronConfiguration.UDAF_FALLBACK_ENABLE.get()) {
           udaf match {
             case _: DeclarativeAggregate =>
             case _: TypedImperativeAggregate[_] =>
@@ -1188,9 +1276,9 @@ object NativeConverters extends Logging {
               .setInputSchema(NativeConverters.convertSchema(paramsSchema)))
           aggBuilder.addAllChildren(convertedChildren.keys.asJava)
         } else {
-          throw new NotImplementedError(
-            s"unsupported aggregate expression: (${e.getClass})," +
-              s" set ${AuronConf.UDAF_FALLBACK_ENABLE.key} true to enable UDAF fallbacking")
+          throw new NotImplementedError(s"Unsupported aggregate expression: (${e.getClass})," +
+            s" set ${SparkAuronConfiguration.SPARK_PREFIX}${SparkAuronConfiguration.UDAF_FALLBACK_ENABLE.key}" +
+            s" to true to enable UDAF fallbacking.")
         }
 
     }
@@ -1256,12 +1344,26 @@ object NativeConverters extends Logging {
         pb.PhysicalScalarFunctionNode
           .newBuilder()
           .setName(name)
-          .setFun(pb.ScalarFunction.SparkExtFunctions)
+          .setFun(pb.ScalarFunction.AuronExtFunctions)
           .addAllArgs(
             args.map(expr => convertExprWithFallback(expr, isPruningExpr, fallback)).asJava)
           .setReturnType(convertDataType(dataType)))
     }
 
+  private def buildTimePartExt(
+      name: String,
+      child: Expression,
+      isPruningExpr: Boolean,
+      fallback: Expression => pb.PhysicalExprNode): pb.PhysicalExprNode = {
+    val tzArg: Expression = child.dataType match {
+      case TimestampType =>
+        Literal.create(SQLConf.get.sessionLocalTimeZone, StringType)
+      case _ =>
+        Literal.create(null, StringType)
+    }
+    buildExtScalarFunctionNode(name, Seq(child, tzArg), IntegerType, isPruningExpr, fallback)
+  }
+
   def castIfNecessary(expr: Expression, dataType: DataType): Expression = {
     if (expr.dataType == dataType) {
       return expr
@@ -1296,9 +1398,20 @@ object NativeConverters extends Logging {
       serialized: Array[Byte]): (E with Serializable, S) = {
     Utils.tryWithResource(new ByteArrayInputStream(serialized)) { bis =>
       Utils.tryWithResource(new ObjectInputStream(bis)) { ois =>
-        val expr = ois.readObject().asInstanceOf[E with Serializable]
-        val payload = ois.readObject().asInstanceOf[S with Serializable]
-        (expr, payload)
+        def read(): (E with Serializable, S) = {
+          val expr = ois.readObject().asInstanceOf[E with Serializable]
+          val payload = ois.readObject().asInstanceOf[S with Serializable]
+          (expr, payload)
+        }
+        // Spark TaskMetrics#externalAccums is not thread-safe
+        val taskContext = TaskContext.get()
+        if (taskContext != null) {
+          taskContext.taskMetrics().synchronized {
+            read()
+          }
+        } else {
+          read()
+        }
       }
     }
   }
